/*
 * File Selection - A java utility to browse and manipulate sets of files.
 * Copyright (C) 2013  David I. Bell
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fileselection;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JMenuBar;
import javax.swing.JTable;

import java.awt.BorderLayout;
import java.awt.Dimension;

import fileselection.utilities.Settings;
import fileselection.utilities.SettingsSaverItf;


/**
 * A class which builds and manages a top-level window which shows a
 * table of files, a table of select lists, and a command input area.
 * The two tables are separated by a slider and there is a menu bar
 * at the top of the window.
 */
public class MainWindow implements SettingsSaverItf
{
	/**
	 * The property name for the main window's size.
	 */
	private static final String PROP_MAIN_WINDOW_SIZE =
		"main.window.size";

	/**
	 * The property name for the split divider location.
	 */
	private static final String PROP_MAIN_WINDOW_DIVIDER_POSITION =
		"main.window.divider.position";

	/**
	 * The frame for the display.
	 */
	private JFrame mFrame;

	/**
	 * The file selection manager that we are using.
	 */
	private FileSelectionManager mManager;

	/**
	 * The file list panel.
	 */
	private FileListPanel mFileListPanel;

	/**
	 * The selection list panel.
	 */
	private SelectListPanel mSelectListPanel;

	/**
	 * The command panel.
	 */
	private CommandPanel mCommandPanel;

	/**
	 * The main panel of the frame.
	 */
	private JPanel mMainPanel;

	/**
	 * A split pane for showing the file list and the selection list.
	 */
	private JSplitPane mSplitPane;

	/**
	 * The menu bar at the top of the window.
	 */
	private JMenuBar mMenuBar;

	/**
	 * The object to execute menu actions.
	 */
	private MenuActions mMenuActions;

	/**
	 * The object to execute file actions.
	 */
	private FileActions mFileActions;


	/**
	 * Constructor for the main file selection application window
	 * which accepts the file selection manager which holds all of
	 * the file lists and select lists data.
	 */
	public MainWindow(FileSelectionManager manager)
	{
		//
		// Save the file selection manager.
		//
		mManager = manager;

		//
		// Allocate the top level frame and set its close option.
		//
		mFrame = new JFrame("File Selection");

		mFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		//
		// Create the main panel for the application.
		//
		mMainPanel = new JPanel(new BorderLayout());

		//
		// Set the preferred size from the properties value with a
		// reasonable default if it isn't found.
		//
		Settings settings = Settings.getInstance();

		Dimension dimension = settings.getDimension(
			PROP_MAIN_WINDOW_SIZE, new Dimension(1000, 700));

		mMainPanel.setPreferredSize(dimension);

		//
		// Build the panel holding the file list.
		//
		mFileListPanel = new FileListPanel(this);

		//
		// Build the panel holding the selection list.
		//
		mSelectListPanel = new SelectListPanel(this);

		//
		// Create the split pane holding the two panels.
		//
		mSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, false,
			mFileListPanel, mSelectListPanel);

		//
		// Allocate the menu actions object.
		//
		mMenuActions = new MenuActions(this);

		//
		// Allocate the file actions object.
		//
		mFileActions = new FileActions(this);

		//
		// Get the menu controller and build the menu bar.
		//
		MenuController menuController = new MenuController(this);

		mMenuBar = menuController.buildMenuBar();

		//
		// Allocate the command panel.
		//
		mCommandPanel = new CommandPanel(this);

		//
		// Add the menu bar at the top of the main panel, the command
		// panel to the bottom of the main panel, and the split pane
		// to the rest of the center of the panel.
		//
		mMainPanel.add(mMenuBar, BorderLayout.NORTH);
		mMainPanel.add(mSplitPane, BorderLayout.CENTER);
		mMainPanel.add(mCommandPanel, BorderLayout.SOUTH);

		//
		// Put the main panel into the frame.
		//
		mFrame.setContentPane(mMainPanel);

		//
		// Register ourself with the settings object so we can save
		// the final window size when we exit.
		//
		settings.registerSettingsSaver(this);

		//
		// Show the window while setting the divider location.
		//
		mFrame.pack();

		int position = settings.getInteger(
			PROP_MAIN_WINDOW_DIVIDER_POSITION, 66);

		if (position < 5)
			position = 5;

		if (position > 95)
			position = 95;

		mSplitPane.setDividerLocation(((float) position) / 100.0);

		mFrame.setVisible(true);
	}


	/**
	 * Return the FileSelectionManager which we are using.
	 */
	public FileSelectionManager getManager()
	{
		return mManager;
	}


	/**
	 * Return the JFrame which holds the top level application window.
	 */
	public JFrame getJFrame()
	{
		return mFrame;
	}


	/**
	 * Return the JTable for the FileInfo table.
	 */
	public JTable getFileInfoJTable()
	{
		return mFileListPanel.getFileInfoJTable();
	}


	/**
	 * Return the JTable for the SelectList table.
	 */
	public JTable getSelectListJTable()
	{
		return mSelectListPanel.getSelectListJTable();
	}


	/**
	 * Return the menu actions object.
	 */
	public MenuActions getMenuActions()
	{
		return mMenuActions;
	}


	/**
	 * Return the file actions object.
	 */
	public FileActions getFileActions()
	{
		return mFileActions;
	}


	/**
	 * Set the column widths of the FileInfo table to their preferred size.
	 */
	public void setFileInfoColumnWidths()
	{
		mFileListPanel.setColumnWidths();
	}


	/**
	 * Store the current column widths of the FileInfo table as their
	 * preferred size.
	 */
	public void storeFileInfoColumnWidths()
	{
		mFileListPanel.storeColumnWidths();
	}


	/**
	 * The implementation of SettingsSaver which lets us save our
	 * state in the properties file before exiting. 
	 */
	@Override
	public void saveSettings(Settings settings)
	{
		//
		// Save the current size of the main window.
		//
		settings.setValue(PROP_MAIN_WINDOW_SIZE, mMainPanel.getSize());

		//
		// Calculate and save the split pane divider position.
		//
		double floatPosition = ((float) mSplitPane.getDividerLocation()) *
			100.0 /	mSplitPane.getHeight();

		int position = (int) (Math.round(floatPosition + 0.5));

		settings.setValue(PROP_MAIN_WINDOW_DIVIDER_POSITION, position);
	}
}
