/*
 * File Selection - A java utility to browse and manipulate sets of files.
 * Copyright (C) 2013  David I. Bell
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fileselection;

import java.awt.BorderLayout;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.util.StringTokenizer;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import javax.swing.event.MouseInputAdapter;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableRowSorter;

import fileselection.dialog.SelectListActionsPopupMenu;

import fileselection.utilities.Settings;
import fileselection.utilities.SettingsSaverItf;


/**
 * A panel which contains a select list table along with controlling
 * buttons and status labels.
 */
public class SelectListPanel extends JPanel implements SettingsSaverItf
{
	private static final long serialVersionUID = 1L;

	/**
	 * Our property value for the column settings.
	 */
	private static final String PROP_SELECT_LIST_TABLE_COLUMNS =
		"select.list.table.columns";

	/**
	 * The main window of the application.
	 */
	private final MainWindow mMainWindow;

	/**
	 * Our controlling select list manager.
	 */
	private final FileSelectionManager mManager;

	/**
	 * The settings class.
	 */
	private final Settings mSettings;

	/**
	 * The table used for showing the select lists.
	 */
	private JTable mTable;

	/**
	 * Our specialised table model for the select list table.
	 */
	private SelectListTableModel mModel;

	/**
	 * The scroll pane which holds the select list table.
	 */
	private JScrollPane mScrollPane;


	/**
	 * Constructor which accepts the main window.
	 */
	public SelectListPanel(MainWindow mainWindow)
	{
		super(new BorderLayout());

		mMainWindow = mainWindow;
		mManager = mainWindow.getManager();

		mSettings = Settings.getInstance();

		//
		// Create the sub-panels to be contained within this one.
		//
		createSelectListPanel();

		//
		// Add the sub-panels into this panel.
		//
		add(mScrollPane, BorderLayout.CENTER);

		//
		// Register ourself with the settings object so that we can
		// save our column definitions on exit.
		//
		mSettings.registerSettingsSaver(this);
	}


	/**
	 * Create the select list sub-panel.
	 */
	private void createSelectListPanel()
	{
		//
		// Get our specialised table model from the file selection manager.
		//
		mModel = mManager.getSelectListTableModel();

		//
		// Allocate the table using this table model.
		//
		mTable = new JTable(mModel);

		//
		// Set our own renderer for the table.
		//
		SelectListTableCellRenderer renderer =
			new SelectListTableCellRenderer(mModel);

		mTable.setDefaultRenderer(String.class, renderer);
		mTable.setDefaultRenderer(Long.class, renderer);
		mTable.setDefaultRenderer(Integer.class, renderer);

		mTable.setAutoResizeMode(JTable.AUTO_RESIZE_NEXT_COLUMN);
		mTable.setRowSelectionAllowed(true);

		//
		// Create a sorter for the table.
		//
		mTable.setRowSorter(new TableRowSorter<SelectListTableModel>(mModel));

		//
		// Add a mouse listener so we can do pop-up menus on the
		// selected rows.
		//
		mTable.addMouseListener(new MouseInputAdapter()
		{
			@Override
			public void mouseClicked(MouseEvent e)
			{
				handleSelectListTableMouseEvent(e);
			}
		});

		//
		// Configure the table columns and then set their widths.
		//
		configureColumns();

		setColumnWidths();

		//
		// Wrap the table inside of a scroll pane.
		//
		mScrollPane = new JScrollPane(mTable);
	}


	/**
	 * Handle a mouse event from our select list table.
	 * We only care about events with the META key down so that
	 * we don't interfere with the row selection table listener.
	 */
	private void handleSelectListTableMouseEvent(MouseEvent e)
	{
		//
		// If this is not a right-click or equivalent then ignore it.
		//
		int modifiers = e.getModifiersEx();

		if ((modifiers & MouseEvent.META_DOWN_MASK) == 0)
			return;

		//
		// Get the viewed row and column in the table where the mouse
		// event occurred.
		//
		Point point = e.getPoint();

		int row = mTable.rowAtPoint(point);
		int column = mTable.columnAtPoint(point);

		if ((row < 0) || (column < 0))
			return;

		//
		// Convert the viewed row number to the table model row
		// number to take into account sorting or filtering.
		//
		row = mTable.convertRowIndexToModel(row);

		if (row < 0)
			return;

		//
		// Get the select list for the row.
		//
		AbstractSelectList selectList = mModel.getSelectList(row);

		//
		// Get the select list popup class.
		//
		SelectListActionsPopupMenu popup =
			new SelectListActionsPopupMenu(mMainWindow, mTable);

		//
		// Put the selected select list into the popup.
		//
		popup.setSelectList(selectList);

		//
		// Show the popup near the mouse event.
		//
		popup.show(point);
	}


	/**
	 * Return the JTable which holds the select list rows.
	 */
	public JTable getSelectListJTable()
	{
		return mTable;
	}


	/**
	 * Load the settings for the columns from the property file.
	 * This defines the widths of the columns.
	 */
	private void configureColumns()
	{
		//
		// Get the column definition setting if present.
		//
		String columnList =
			mSettings.getString(PROP_SELECT_LIST_TABLE_COLUMNS, null);

		//
		// If the property value is missing then do nothing.
		//
		if (columnList == null)
			return;

		//
		// Parse the column info items.
		// The column info items are separated by spaces.
		//
		StringTokenizer token = new StringTokenizer(columnList, " ");

		while (token.hasMoreTokens())
			configureColumn(token.nextToken());
	}


	/**
	 * Attempt to configure one column for the select list table.
	 * This uses a definition string value of the form:
	 *		name:width
	 * Illegal or unknown columns are ignored.
	 */
	private void configureColumn(String definition)
	{
		//
		// Break up the definition into two strings.
		//
		StringTokenizer token = new StringTokenizer(definition, ":");

		if (!token.hasMoreTokens())
			return;

		String name = token.nextToken();

		if (!token.hasMoreTokens())
			return;

		String widthString = token.nextToken();

		//
		// Parse the width.
		//
		int width = 0;

		try
		{
			width = Integer.parseInt(widthString);
		}
		catch (Exception e)
		{
			return;
		}

		//
		// Try to set the indicated column's preferred width.
		//
		int index = mModel.getColumnIdIndex(name);

		if (index >= 0)
			mModel.setPreferredColumnWidth(index, width);
	}


	/**
	 * Set the column widths to their preferred values.
	 */
	private void setColumnWidths()
	{
		//
		// Get the column model for the table and the column count.
		//
		TableColumnModel columnModel = mTable.getColumnModel();
		int columnCount = mTable.getColumnCount();

		//
		// Loop over all of the columns setting their preferred sizes.
		//
		for (int columnIndex = 0; columnIndex < columnCount; columnIndex++)
		{
			//
			// Get the column for this index.
			//
			TableColumn column = columnModel.getColumn(columnIndex);

			//
			// Ask the table model for its preferred width of the column.
			// Note we have to use the model index here since the columns
			// can be reordered by the user.
			//
			int modelIndex = column.getModelIndex();

			int width = mModel.getPreferredColumnWidth(modelIndex);

			//
			// If the width is positive then set it.
			//
			if (width > 0)
			{
				column.setPreferredWidth(width);
				column.setWidth(width);
			}
		}
	}


	/**
	 * Implementation of SettingsSaver method to save our settings
	 * before exiting.  For us, this is the column names and widths
	 * for the select list table.
	 */
	@Override
	public void saveSettings(Settings settings)
	{
		//
		// Get the column model.
		//
		TableColumnModel columnModel = mTable.getColumnModel();

		int columnCount = columnModel.getColumnCount();

		StringBuilder buffer = new StringBuilder();

		//
		// Iterate over all of the columns.
		//
		for (int columnIndex = 0; columnIndex < columnCount; columnIndex++)
		{
			//
			// Get the table column object for this column.
			//
			TableColumn tableColumn = columnModel.getColumn(columnIndex);

			//
			// Get the actual width of this column.
			//
			int width = tableColumn.getWidth();

			//
			// Get the name of the column.
			// Note this uses the model index since the user can rearrange
			// the columns.
			//
			int modelIndex = tableColumn.getModelIndex();

			String columnId = mModel.getColumnId(modelIndex);

			//
			// Append this information to the buffer.
			//
			if (buffer.length() > 0)
				buffer.append(' ');

			buffer.append(columnId);
			buffer.append(':');
			buffer.append(width);
		}

		String columnInfo = buffer.toString();

		mSettings.setValue(PROP_SELECT_LIST_TABLE_COLUMNS, columnInfo);
	}
}