/*
 * File Selection - A java utility to browse and manipulate sets of files.
 * Copyright (C) 2013  David I. Bell
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fileselection;

import java.awt.Component;
import java.awt.Color;
import java.awt.Rectangle;

import javax.swing.BorderFactory;
import javax.swing.JTable;
import javax.swing.JLabel;
import javax.swing.SwingConstants;
import javax.swing.table.TableCellRenderer;


/**
 * A cell renderer for the cells in the select list table.
 * This is needed simply to apply colored backgrounds to the rows.
 */
public class SelectListTableCellRenderer implements TableCellRenderer
{
	/**
	 * The template label used for rendering.
	 */
	private JLabel mLabelRendererComponent;

	/**
	 * Our owning table.
	 */
	private JTable mTable;


	/**
	 * Constructor.
	 */
	public SelectListTableCellRenderer(SelectListTableModel model)
	{
		//
		// Allocate the template components used for rendering.
		// First allocate an overridden version of JLabel which turns
		// some methods into no-ops (like DefaultTableCellRender does).
		//
		mLabelRendererComponent = new JLabel()
		{
			private static final long serialVersionUID = 1L;

			@Override
			public void validate() {}

			@Override
			public void revalidate() {}

			@Override
			public void repaint(long tm, int x, int y, int width, int height) {}

			@Override
			public void repaint(Rectangle r) {}

			@Override
			public void firePropertyChange(String propertyName, boolean oldValue, boolean newValue) {}
		};

		mLabelRendererComponent.setOpaque(true);

		mLabelRendererComponent.setBorder(
			BorderFactory.createEmptyBorder(5, 5, 5, 5));
	}


	/**
	 * TableCellRenderer method which returns a component used to stamp
	 * out the value of a cell being rendered.
	 */
	@Override
	public Component getTableCellRendererComponent(JTable table,
		Object value, boolean isSelected, boolean hasFocus,
		int row, int column)
	{
		mTable = table;

		//
		// Switch on the type of value to handle it.
		//
		Component component;

		if (value instanceof String)
			component = getComponent((String) value);
		else if (value instanceof Integer)
			component = getComponent(((Integer) value).longValue());
		else if (value instanceof Long)
			component = getComponent(((Long) value).longValue());
		else
			component = getComponent(value.toString());

		//
		// Get the select list for the row being painted.
		// Take into account any sorting that may be set.
		//
		SelectListTableModel model = (SelectListTableModel) table.getModel();

		row = table.convertRowIndexToModel(row);

		AbstractSelectList selectList = model.getSelectList(row);

		//
		// Prepare to set the foreground and background colors.
		// First see if there is an associated background color
		// for this select list.
		//
		Color foreground;
		Color background = selectList.getColor();

		//
		// Switch on whether the row is selected.
		//
		if (isSelected)
		{
			foreground = table.getSelectionForeground();

			//
			// If there was an associated background color for the select
			// list then slightly darken it to indicate the selection,
			// otherwise use the normal table's selection background color.
			//
			if (background != null)
				background = background.darker();
			else
				background = table.getSelectionBackground();
		}
		else
		{
			foreground = table.getForeground();

			//
			// If there isn't an associated background color for the file
			// then use the normal table's background color.
			//
			if (background == null)
				background = table.getBackground();
		}

		component.setForeground(foreground);
		component.setBackground(background);

		//
		// Return the component ready for use.
		//
		return component;
	}


	/**
	 * Return a JLabel which holds the indicated long value.
	 * This is right justified.
	 */
	private Component getComponent(long value)
	{
		//
		// Set the text value of our rendering label and return it.
		//
		mLabelRendererComponent.setFont(mTable.getFont());
		mLabelRendererComponent.setHorizontalAlignment(SwingConstants.RIGHT);
		mLabelRendererComponent.setText(String.valueOf(value));

		return mLabelRendererComponent;
	}


	/**
	 * Return a JLabel which holds the indicated string value.
	 */
	private Component getComponent(String value)
	{
		//
		// Set the text value of our rendering label and return it.
		//
		mLabelRendererComponent.setFont(mTable.getFont());
		mLabelRendererComponent.setHorizontalAlignment(SwingConstants.LEFT);
		mLabelRendererComponent.setText(value);

		return mLabelRendererComponent;
	}
}
