/*
 * File Selection - A java utility to browse and manipulate sets of files.
 * Copyright (C) 2013  David I. Bell
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fileselection.calculation;

import java.util.BitSet;

import fileselection.ManualSelectList;


/**
 * A class to hold a calculation tree which was parsed from an expression
 * and to perform calculations on it. 
 */
public class CalculationTree
{
	/**
	 * The expression which was parsed to provide the node tree.
	 */
	private final String mExpression;

	/**
	 * The root calculation node of the tree.
	 */
	private final CalculationNode mRootNode;

	/**
	 * An error message generated by the last execution error.
	 */
	private String mErrorMessage;


	/**
	 * Constructor which accepts the expression which was parsed and
	 * the resulting root node of the calculation tree.
	 */
	public CalculationTree(String expression, CalculationNode rootNode)
	{
		mExpression = expression;
		mRootNode = rootNode;
	}


	/**
	 * Evaluate the calculation tree to produce a resulting bit set
	 * representing the files which match the expression.  Returns null
	 * if the evaluation failed.  In this case, getErrorMessage can be
	 * called to obtain the reason for the failure.
	 */
	public BitSet evaluateBitSet()
	{
		//
		// Catch exceptions so that we can generate an error message.
		//
		try
		{
			BitSet bitSet = mRootNode.evaluateBitSet();

			mErrorMessage = null;

			return bitSet;
		}
		catch (CalculationException e)
		{
			mErrorMessage = e.getMessage();

			return null;
		}
		catch (Exception e)
		{
			e.printStackTrace();
			mErrorMessage = "Exception during evaluation: " + e;

			return null;
		}
	}


	/**
	 * Return the root calculation node of the tree. 
	 */
	public CalculationNode getRootNode()
	{
		return mRootNode;
	}


	/**
	 * Return the manual select list that is to be assigned to.
	 * This is null if the expression is not an assignment.
	 */
	public ManualSelectList getAssignmentSelectList()
	{
		switch (mRootNode.mOpcode)
		{
			case CalculationNode.OP_ASSIGN:
			case CalculationNode.OP_ASSIGN_OR:
			case CalculationNode.OP_ASSIGN_AND:
				return (ManualSelectList) mRootNode.mFirstChild.mSelectList;

			default:
				return null;
		}
	}


	/**
	 * Return the expression that was used to build the calculation tree.
	 */
	public String getExpression()
	{
		return mExpression;
	}


	/**
	 * Return the error message describing the reason for the last
	 * calculation failure.  Returns null if the calculation was successful.
	 */
	public String getErrorMessage()
	{
		return mErrorMessage;
	}
}
