/*
 * File Selection - A java utility to browse and manipulate sets of files.
 * Copyright (C) 2013  David I. Bell
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fileselection.dialog;

import java.awt.Component;
import java.awt.BorderLayout;
import java.util.List;

import javax.swing.JPanel;
import javax.swing.JOptionPane;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JLabel;

import fileselection.FileInfoColumnItf;
import fileselection.FileInfoItem;
import fileselection.FileSelectionManager;
import fileselection.MainWindow;
import fileselection.AbstractSelectList;
import fileselection.utilities.Utils;


/**
 * Class to build and show an option dialog that lets the user add
 * new columns to the FileInfo table.  The columns are of the
 * FileInfoColumn class.  A column is not added if it is already
 * being shown in the table.
 */
public class AddFileInfoColumnsDialog
{
	/**
	 * The FileInfoColumn objects that were selected.
	 */
	private FileInfoColumnItf[] mColumnObjects;

	/**
	 * The file selection manager.
	 */
	private final FileSelectionManager mManager;

	/**
	 * The SelectList JList.
	 */
	private JList<AbstractSelectList> mSelectListList;

	/**
	 * The FileInfoItem JList.
	 */
	private JList<FileInfoItem> mFileInfoItemList;

	/**
	 * Our parent component.
	 */
	private final Component mParent;


	/**
	 * Constructor which sets the main window.
	 */
	public AddFileInfoColumnsDialog(MainWindow mainWindow)
	{
		//
		// Extract the values we need.
		//
		mManager = mainWindow.getManager();
		mParent = mainWindow.getJFrame();
	}


	/**
	 * Show the Add FileInfoColumn Dialog.
	 * Returns true if the user used the OK button.
	 * After this the column to be added can be obtained.
	 */
	public boolean show()
	{
		//
		// Allocate the panel holding the dialog contents.
		//
		JPanel mainPanel = new JPanel();

		//
		// Build and add the file info and select list lists
		// side by side.
		//
		mainPanel.add(buildFileInfoItemPanel());
		mainPanel.add(buildSelectListPanel());

		//
		// Show the dialog and get which button was used.
		//
		int result = JOptionPane.showConfirmDialog(mParent, mainPanel,
			"Add Columns", JOptionPane.OK_CANCEL_OPTION);

		//
		// If the dialog was cancelled then return false.
		//
		if (result != JOptionPane.OK_OPTION)
			return false;

		//
		// Save the selected objects and return success.
		//
		List<FileInfoItem> fileInfoItemList = mFileInfoItemList.getSelectedValuesList();
		List<AbstractSelectList> selectListList = mSelectListList.getSelectedValuesList();

		int fileInfoItemSize = fileInfoItemList.size();
		int selectListListSize = selectListList.size();

		mColumnObjects = new FileInfoColumnItf[fileInfoItemSize + selectListListSize];

		for (int i = 0; i < fileInfoItemSize; i++)
			mColumnObjects[i] = fileInfoItemList.get(i);

		for (int i = 0; i < selectListListSize; i++)
		{
			mColumnObjects[fileInfoItemSize + i] = selectListList.get(i);
		}

		return true;
	}


	/**
	 * Build a panel showing the available file into items.
	 * This has a label at the top and a scrollable list below that.
	 */
	private JPanel buildFileInfoItemPanel()
	{
		//
		// Allocate a panel with a border layout.
		//
		JPanel panel = new JPanel(new BorderLayout());

		//
		// At a label at the top of the layout.
		//
		panel.add(new JLabel("File Status"), BorderLayout.NORTH);

		//
		// Get an array of all of the File Info Items.
		//
		FileInfoItem[] fileInfoItemArray =
			FileInfoItem.getFileInfoItemArray();

		//
		// Build a list to show those objects wrapped in a scroll pane.
		//
		mFileInfoItemList = new JList<FileInfoItem>(fileInfoItemArray);
		mFileInfoItemList.setAutoscrolls(true);

		mFileInfoItemList.setVisibleRowCount(
			Utils.getListVisibleRowCount(fileInfoItemArray.length));

		JScrollPane scrollPane = new JScrollPane(mFileInfoItemList);

		//
		// Add the scroll pane to the rest of the layout.
		//
		panel.add(scrollPane, BorderLayout.CENTER);

		//
		// Return the panel.
		//
		return panel;
	}


	/**
	 * Build a panel showing the available select lists.
	 * This has a label at the top and a scrollable list below that.
	 */
	private JPanel buildSelectListPanel()
	{
		//
		// Allocate a panel with a border layout.
		//
		JPanel panel = new JPanel(new BorderLayout());

		//
		// At a label at the top of the layout.
		//
		panel.add(new JLabel("Select Lists"), BorderLayout.NORTH);

		//
		// Build the full select list list.
		//
		mSelectListList = Utils.buildSelectListList(mManager, true);

		//
		// Wrap the list in a scroll pane.
		//
		JScrollPane scrollPane = new JScrollPane(mSelectListList);

		//
		// Add the scroll pane to the rest of the layout.
		//
		panel.add(scrollPane, BorderLayout.CENTER);

		//
		// Return the panel.
		//
		return panel;
	}


	/**
	 * Return the array of FileInfoColumn objects to be added.
	 */
	public FileInfoColumnItf[] getColumnObjects()
	{
		return mColumnObjects;
	}
}
