/*
 * File Selection - A java utility to browse and manipulate sets of files.
 * Copyright (C) 2013  David I. Bell
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fileselection.dialog;

import java.awt.Component;
import java.awt.Point;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;

import fileselection.ManualSelectList;
import fileselection.FileInfo;
import fileselection.FileActions;
import fileselection.FileSelectionManager;
import fileselection.MainWindow;


/**
 * A class to run and process a popup menu for selected files.
 * Many of the actions are executed by the FileActions class.
 * We have a fixed set of menu items, along with an optional set of
 * menu items which performing adding or removing of files from specific
 * select lists.
 */
public class FileActionsPopupMenu implements ActionListener
{
	/**
	 * The popup menu.
	 */
	private JPopupMenu mMenu;

	/**
	 * The menu item for viewing a file.
	 */
	private JMenuItem mViewFileMenuItem;

	/**
	 * The menu item for viewing a file as text.
	 */
	private JMenuItem mViewFileAsTextMenuItem;

	/**
	 * The menu item for viewing a file as binary.
	 */
	private JMenuItem mViewFileAsBinaryMenuItem;

	/**
	 * The menu item for opening a file using the associated program.
	 */
	private JMenuItem mOpenFileMenuItem;

	/**
	 * The menu item for using the specified program on a file.
	 */
	private JMenuItem mRunProgramOnFileMenuItem;

	/**
	 * The menu item for updating the file status.
	 */
	private JMenuItem mUpdateFileStatusMenuItem;

	/**
	 * The menu item for clearing the "is changed" file status.
	 */
	private JMenuItem mClearIsChangedMenuItem;

	/**
	 * The menu item for adding the file to a select list.
	 */
	private JMenuItem mAddToSelectListMenuItem;

	/**
	 * The menu item for removing the file from a select list.
	 */
	private JMenuItem mRemoveFromSelectListMenuItem;

	/**
	 * An array of optional menu items for adding to specific select lists.
	 */
	private JMenuItem[] mAddToSelectListMenuItemArray;

	/**
	 * An array of optional menu items for removing from specific select lists.
	 */
	private JMenuItem[] mRemoveFromSelectListMenuItemArray;

	/**
	 * The file info which we are acting on.
	 */
	private FileInfo mFileInfo;

	/**
	 * The file actions class.
	 */
	private final FileActions mActions;

	/**
	 * The file selection manager.
	 */
	private final FileSelectionManager mManager;

	/**
	 * The main window.
	 */
	private final MainWindow mMainWindow;

	/**
	 * The parent window.
	 */
	private final Component mParent;


	/**
	 * Constructor which sets the main window and our parent component.
	 */
	public FileActionsPopupMenu(MainWindow mainWindow, Component parent)
	{
		//
		// Extract the values which we need.
		//
		mMainWindow = mainWindow;
		mManager = mainWindow.getManager();
		mActions = mainWindow.getFileActions();

		if (parent == null)
			mParent = mainWindow.getJFrame();
		else
			mParent = parent;
	}


	/**
	 * Set the single file info which our actions are to act on.
	 */
	public void setFileInfo(FileInfo fileInfo)
	{
		mFileInfo = fileInfo;
	}


	/**
	 * Build and show the popup menu near the specified point showing
	 * the list file actions.  The first set of actions are constant.
	 * Following these there can be optional actions which add or remove
	 * the file from specific select lists.
	 */
	public void show(Point point)
	{
		//
		// Create the popup menu.
		//
		mMenu = new JPopupMenu("File Actions");

		//
		// Add the fixed menu items.
		//
		mViewFileMenuItem = buildMenuItem("View File...");
		mViewFileAsTextMenuItem = buildMenuItem("View File as Text...");
		mViewFileAsBinaryMenuItem = buildMenuItem("View File as Binary...");
		mOpenFileMenuItem = buildMenuItem("Open File...");
		mRunProgramOnFileMenuItem = buildMenuItem("Run Program on File...");
		mUpdateFileStatusMenuItem = buildMenuItem("Update File Status");
		mClearIsChangedMenuItem = buildMenuItem("Clear \"is Changed\" Status");
		mAddToSelectListMenuItem = buildMenuItem("Add to Select List...");
		mRemoveFromSelectListMenuItem = buildMenuItem("Remove from Select List...");

		//
		// Add the optional adding select list menu items.
		//
		buildAddingSelectListItems();

		//
		// Add the optional removing select list menu items.
		//
		buildRemovingSelectListItems();

		//
		// Make the menu visible at the correct spot.
		//
		mMenu.pack();

		mMenu.show(mParent, point.x, point.y);
	}


	/**
	 * Build menu items for the select lists which have been specified
	 * as having files easily added to them.
	 */
	private void buildAddingSelectListItems()
	{
		//
		// Get the array of the currently defined adding select lists.
		//
		ManualSelectList[] selectListArray =
			mActions.getAddToSelectListArray();

		//
		// If the array is non-empty then add a separator before the
		// optional add menu items.
		//
		if (selectListArray.length > 0)
			mMenu.addSeparator();

		//
		// Allocate an array for the menu items.
		//
		int count = selectListArray.length;

		mAddToSelectListMenuItemArray = new JMenuItem[count];

		//
		// Build the menu items into the array.
		//
		for (int index = 0; index < count; index++)
		{
			String label = "Add to Select List \"" +
				selectListArray[index].getName() + "\"";

			mAddToSelectListMenuItemArray[index] = buildMenuItem(label);
		}
	}


	/**
	 * Build menu items for the select lists which have been specified
	 * as having files easily removed from them.
	 */
	private void buildRemovingSelectListItems()
	{
		//
		// Get the array of the currently defined removing select lists.
		//
		ManualSelectList[] selectListArray =
			mActions.getRemoveFromSelectListArray();

		//
		// If the array is non-empty then add a separator before the
		// optional remove menu items.
		//
		if (selectListArray.length > 0)
			mMenu.addSeparator();

		//
		// Allocate an array for the menu items.
		//
		int count = selectListArray.length;

		mRemoveFromSelectListMenuItemArray = new JMenuItem[count];

		//
		// Build the menu items into the array.
		//
		for (int index = 0; index < count; index++)
		{
			String label = "Remove from Select List \"" +
				selectListArray[index].getName() + "\"";

			mRemoveFromSelectListMenuItemArray[index] = buildMenuItem(label);
		}
	}


	/**
	 * Build a menu item and add it to the popup menu.
	 */
	private JMenuItem buildMenuItem(String label)
	{
		JMenuItem menuItem = new JMenuItem(label);

		menuItem.addActionListener(this);

		mMenu.add(menuItem);

		return menuItem;
	}


	/**
	 * Implementation of ActionListener method called when a menu item
	 * is selected.
	 */
	@Override
	public void actionPerformed(ActionEvent e)
	{
		//
		// Remove the menu.
		//
		mMenu.setVisible(false);

		//
		// Make sure there is a file given to us.
		//
		if (mFileInfo == null)
		{
			mActions.showOpenFileWarning("No file set for action");

			return;
		}

		//
		// Set the selected file into the actions class.
		//
		mActions.setCurrentFile(mFileInfo);

		//
		// Switch on the menu item to do the appropriate action.
		//
		Object source = e.getSource();

		if (source == mViewFileMenuItem)
			viewFileAction();
		else if (source == mViewFileAsTextMenuItem)
			viewFileAsTextAction();
		else if (source == mViewFileAsBinaryMenuItem)
			viewFileAsBinaryAction();
		else if (source == mOpenFileMenuItem)
			mActions.openFileAction();
		else if (source == mRunProgramOnFileMenuItem)
			mActions.runProgramOnFileAction();
		else if (source == mUpdateFileStatusMenuItem)
			mActions.updateFileStatusAction();
		else if (source == mClearIsChangedMenuItem)
			mActions.clearIsChangedAction();
		else if (source == mAddToSelectListMenuItem)
			mActions.addToSelectListAction();
		else if (source == mRemoveFromSelectListMenuItem)
			mActions.removeFromSelectListAction();
		else if (!checkOptionalSelectListAction(source))
			mActions.showOpenFileWarning("Unhandled action");

		//
		// Update anything that the action changed.
		//
		mManager.firePendingRowUpdates();
	}


	/**
	 * Check our adding and removing arrays for the given source object
	 * and perform the adding or removing on the matching one.
	 * Returns true if the source was one of our added menu items.
	 */
	private boolean checkOptionalSelectListAction(Object source)
	{
		if (checkOptionalAddToSelectListAction(source))
			return true;

		if (checkOptionalRemoveFromSelectListAction(source))
			return true;

		return false;
	}


	/**
	 * Check our adding menu item array for the given source object and
	 * perform the adding on the matching one.  Returns true if the source
	 * was one of our added items.
	 */
	private boolean checkOptionalAddToSelectListAction(Object source)
	{
		//
		// Search for a matching menu item in the adding menu item array.
		//
		for (int index = 0; index < mAddToSelectListMenuItemArray.length;
			index++)
		{
			if (mAddToSelectListMenuItemArray[index] == source)
			{
				//
				// Add the file to the corresponding select list
				// and return success.
				//
				mActions.addFileToSelectListAction(index);

				return true;
			}
		}

		return false;
	}


	/**
	 * Check our removing menu item array for the given source object and
	 * perform the removing on the matching one.  Returns true if the source
	 * was one of our removal items.
	 */
	private boolean checkOptionalRemoveFromSelectListAction(Object source)
	{
		//
		// Search for a matching menu item in the removing menu item array.
		//
		for (int index = 0; index < mRemoveFromSelectListMenuItemArray.length;
			index++)
		{
			if (mRemoveFromSelectListMenuItemArray[index] == source)
			{
				//
				// Remove the file from the corresponding select list
				// and return success.
				//
				mActions.removeFileFromSelectListAction(index);

				return true;
			}
		}

		return false;
	}


	/**
	 * Action to view a file in our own quick viewer as text or binary
	 * according to the file's extension.
	 */
	private void viewFileAction()
	{
		ViewFileDialog dialog = new ViewFileDialog(mMainWindow, mParent);

		dialog.setFileInfo(mFileInfo);

		dialog.show(ViewFileDialog.VIEW_DEFAULT);
	}


	/**
	 * Action to view a file in our own quick viewer as text.
	 */
	private void viewFileAsTextAction()
	{
		ViewFileDialog dialog = new ViewFileDialog(mMainWindow, mParent);

		dialog.setFileInfo(mFileInfo);

		dialog.show(ViewFileDialog.VIEW_TEXT);
	}


	/**
	 * Action to view a file in our own quick viewer as binary.
	 */
	private void viewFileAsBinaryAction()
	{
		ViewFileDialog dialog = new ViewFileDialog(mMainWindow, mParent);

		dialog.setFileInfo(mFileInfo);

		dialog.show(ViewFileDialog.VIEW_BINARY);
	}
}
