/*
 * File Selection - A java utility to browse and manipulate sets of files.
 * Copyright (C) 2013  David I. Bell
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fileselection.dialog;

import java.net.URL;

import java.awt.Component;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Color;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JEditorPane;
import javax.swing.JScrollPane;

import fileselection.MainWindow;

import fileselection.utilities.Utils;


/**
 * Class to build and show an option dialog which shows a formatted
 * help file to the user for the specified topic.
 */
public class HelpDialog implements ActionListener
{
	/**
	 * The window frame.
	 */
	private JFrame mFrame;

	/**
	 * The topic which we are displaying help for.
	 */
	private String mTopic;

	/**
	 * Our parent component.
	 */
	private final Component mParent;


	/**
	 * Constructor which sets the main window.
	 */
	public HelpDialog(MainWindow mainWindow)
	{
		//
		// Extract the values we need.
		//
		mParent = mainWindow.getJFrame();
	}


	/**
	 * Show the help information for the specified topic.
	 * This does not wait for the dialog to complete.
	 */
	public void show(String topic)
	{
		//
		// Default the topic if it is empty.
		//
		if ((topic == null) || (topic.length() == 1))
			topic = "Overview";

		//
		// Capitalise the topic.
		//
		mTopic = topic.substring(0, 1).toUpperCase() + topic.substring(1);

		//
		// Try to build the panel containing the help topic.
		//
		JPanel displayPanel = buildDisplayPanel();

		//
		// If the help was not found then complain and return.
		//
		if (displayPanel == null)
		{
			JOptionPane.showMessageDialog(mParent,
				"Help not found for topic \"" + topic + "\"",
				"Help Not Found", JOptionPane.ERROR_MESSAGE);

			return;
		}

		//
		// OK, we have the help panel built.
		// Allocate the frame to show it in and set its close option.
		//
		mFrame = new JFrame(mTopic + " Help");

		mFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);

		//
		// Allocate the main panel.
		//
		JPanel mainPanel = new JPanel(new BorderLayout());

		//
		// Build the options panel and add both panels to the main panel.
		//
		mainPanel.add(displayPanel, BorderLayout.CENTER);
		mainPanel.add(buildOptionsPanel(), BorderLayout.SOUTH);

		//
		// Set the content panel.
		//
		mFrame.setContentPane(mainPanel);

		//
		// Show the frame centered on our parent.
		//
		Utils.showCenteredFrame(mFrame, mParent);
	}


	/**
	 * Build and return the display panel showing the help text in a
	 * scrollable region.  Returns null if the help is not available.
	 */
	private JPanel buildDisplayPanel()
	{
		//
		// Build the full resource file name.
		//
		String resourceName = "fileselection/help/Help_" + mTopic + ".html";

		//
		// Try to get the URL for the file using the system class loader.
		// Return null if the help file could not be found.
		//
		URL url = ClassLoader.getSystemResource(resourceName);

		if (url == null)
			return null;

		//
		// Allocate the editor pane and set its properties.
		//
		JEditorPane editorPane = new JEditorPane();

		editorPane.setEditable(false);
		editorPane.setOpaque(true);
		editorPane.setContentType("text/html");

		//
		// Set the page content.
		//
		try
		{
			editorPane.setPage(url);
		}
		catch (Exception e)
		{
			return null;
		}

		//
		// Wrap the editor pane in a scroll pane.
		//
		JScrollPane scrollPane = new JScrollPane(editorPane);

		scrollPane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		scrollPane.setPreferredSize(new Dimension(800, 600));
		scrollPane.setOpaque(true);
		scrollPane.setBackground(Color.white);

		//
		// Allocate a panel to hold the display.
		//
		//
		JPanel panel = new JPanel(new BorderLayout());

		panel.setOpaque(true);
		panel.setBackground(Color.white);

		//
		// Add the scroll pane to the panel and return it.
		//
		panel.add(scrollPane, BorderLayout.CENTER);

		return panel;
	}


	/**
	 * Build the options panel.
	 * This just contains the OK button which dismisses the dialog.
	 */
	private JPanel buildOptionsPanel()
	{
		JPanel panel = new JPanel();

		JButton okButton = new JButton("OK");

		okButton.addActionListener(this);

		panel.add(okButton);

		return panel;
	}


	/**
	 * Implementation of ActionListener method called when the OK
	 * button is pressed.  We just close the dialog. 
	 */
	@Override
	public void actionPerformed(ActionEvent e)
	{
		//
		// Dispose of the dialog.
		//
		if (mFrame != null)
			mFrame.dispose();

		mFrame = null;
	}
}
