/*
 * File Selection - A java utility to browse and manipulate sets of files.
 * Copyright (C) 2013  David I. Bell
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fileselection.dialog;

import java.util.BitSet;

import javax.swing.BorderFactory;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JFrame;
import javax.swing.JButton;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;

import javax.swing.border.TitledBorder;

import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

import java.awt.Component;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import fileselection.FileSelectionManager;
import fileselection.MainWindow;
import fileselection.FileInfo;
import fileselection.AbstractSelectList;

import fileselection.utilities.Utils;
import fileselection.utilities.TimeValue;
import fileselection.utilities.Settings;


/**
 * A dialog to interactively examine the files in one or more select lists.
 * This complicated dialog has six sections:
 *		The select list panel
 *		The file list panel
 *		The file information panel
 *		The file operation panel
 *		The file view panel
 *		The close button
 */
public class InteractiveFileDialog
	implements ActionListener, ListSelectionListener, KeyListener,
	ViewFileListenerItf
{
	/**
	 * Property name for the interactive window size.
	 */
	private static final String PROP_INTERACTIVE_WINDOW_SIZE =
		"interactive.window.size";

	/**
	 * Our frame for the dialog.
	 */
	private JFrame mFrame;

	/**
	 * The main panel for the dialog.
	 */
	private JPanel mMainPanel;

	/**
	 * The panel which holds the file list.
	 */
	private JPanel mFileListPanel;

	/**
	 * The normal titled border for the file list panel.
	 */
	private TitledBorder mFileListBorder;

	/**
	 * A dummy titled border used to work around a swing bug.
	 */
	private TitledBorder mDummyBorder;

	/**
	 * The view file dialog (which we just use use as one of our panels).
	 */
	private ViewFileDialog mViewFileDialog;

	/**
	 * The JList which shows the select list.
	 */
	private JList<AbstractSelectList> mSelectListList;

	/**
	 * The JList which shows the selected file list.
	 */
	private JList<FileInfo> mFileListList;

	/**
	 * The scroll pane which contains the file list.
	 */
	private JScrollPane mFileListScrollPane;

	/**
	 * The button for closing the dialog.
	 */
	private JButton mCloseButton;

	/**
	 * The button to apply the selected select list.
	 */
	private JButton mApplySelectListButton;

	/**
	 * Text field for the finding file names.
	 */
	private JTextField mFindFileNameTextField;

	/**
	 * The button to move to the previous file of a search.
	 */
	private JButton mFindPreviousFileButton;

	/**
	 * The button to move to the next file of a search.
	 */
	private JButton mFindNextFileButton;

	/**
	 * The text field showing full path of the file.
	 */
	private JTextField mFullPathTextField;

	/**
	 * The text field showing the last component of the file name.
	 */
	private JTextField mFileNameTextField;

	/**
	 * The text field showing the depth of the file.
	 */
	private JTextField mFileDepthTextField;

	/**
	 * The text field showing the file's key.
	 */
	private JTextField mFileKeyTextField;

	/**
	 * The text field showing the current file type.
	 */
	private JTextField mFileTypeTextField;

	/**
	 * The text field showing the current file's content type.
	 */
	private JTextField mFileContentTypeTextField;

	/**
	 * The text field showing the current file length.
	 */
	private JTextField mFileLengthTextField;

	/**
	 * The text field showing the current file owner.
	 */
	private JTextField mFileOwnerTextField;

	/**
	 * The text field showing the current file group.
	 */
	private JTextField mFileGroupTextField;

	/**
	 * The text field showing the current file creation time.
	 */
	private JTextField mFileCreationTextField;

	/**
	 * The text field showing the current file last modified time.
	 */
	private JTextField mFileLastModifiedTextField;

	/**
	 * The text field showing the current file last access time.
	 */
	private JTextField mFileLastAccessTextField;

	/**
	 * The text field showing the current file's link value.
	 */
	private JTextField mFileLinkValueTextField;

	/**
	 * The text field showing the current file's checksum.
	 */
	private JTextField mFileChecksumTextField;

	/**
	 * The text field showing the current file's permissions.
	 */
	private JTextField mFilePermissionsTextField;

	/**
	 * The text field showing the access allowed to the file.
	 */
	private JTextField mFileAccessTextField;

	/**
	 * The text field showing the select lists the file is in.
	 */
	private JTextField mFileSelectListsField;

	/**
	 * The select list table.
	 */
	private final JTable mSelectListTable;

	/**
	 * The array of files to be processed.
	 * The index of the values in this array is the same as the values
	 * displayed in the file JList.
	 */
	private FileInfo[] mFileArray = new FileInfo[0];

	/**
	 * The number of files in the array to be processed.
	 */
	private int mFileArraySize;

	/**
	 * The index in the file array of the current file.
	 * This is -1 if there is no current file.
	 */
	private int mFileArrayIndex = -1;

	/**
	 * The current file being processed.
	 * This is null if there is no current file.
	 */
	private FileInfo mCurrentFile;

	/**
	 * Our file selection manager.
	 */
	private final FileSelectionManager mManager;

	/**
	 * The main window.
	 */
	private final MainWindow mMainWindow;

	/**
	 * Our parent component.
	 */
	private final Component mParent;

	/**
	 * The settings class.
	 */
	private final Settings mSettings;


	/**
	 * Constructor which sets the main window.
	 */
	public InteractiveFileDialog(MainWindow mainWindow)
	{
		//
		// Extract the values we need.
		//
		mMainWindow = mainWindow;
		mManager = mainWindow.getManager();
		mParent = mainWindow.getJFrame();
		mSelectListTable = mainWindow.getSelectListJTable();

		//
		// Get the settings for our defaults.
		//
		mSettings = Settings.getInstance();
	}


	/**
	 * Show the Interactive File Dialog.
	 * Unlike most of our dialogs we don't have OK or CANCEL buttons
	 * since operations are performed interactively on the dialog.
	 * All we have is a CLOSE button.
	 */
	public void show()
	{
		//
		// Allocate the frame for the dialog and set its close option.
		//
		mFrame = new JFrame("Interactive File Operation");

		mFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);

		//
		// Allocate the panel holding the message.
		//
		mMainPanel = new JPanel(new BorderLayout());

		//
		// Set the preferred size from the properties value with a
		// reasonable default if it isn't found.
		//
		Dimension dimension = mSettings.getDimension(
			PROP_INTERACTIVE_WINDOW_SIZE, new Dimension(1000, 900));

		mMainPanel.setPreferredSize(dimension);

		//
		// Build and add the two sub-panels which are the main
		// message panel and the options panel containing the
		// close button.
		//
		mMainPanel.add(buildMessagePanel(), BorderLayout.CENTER);
		mMainPanel.add(buildOptionsPanel(), BorderLayout.SOUTH);

		//
		// Set the initial values of the file information fields.
		//
		showCurrentFile();

		//
		// Put the content pane into the frame.
		//
		mFrame.setContentPane(mMainPanel);

		//
		// Show the frame centered on our parent.
		//
		Utils.showCenteredFrame(mFrame, mParent);
	}


	/**
	 * Build the message panel containing the two main sub-panels.
	 * These two panels are vertically aligned.  Together they
	 * contain everything except for the options panel.
	 */
	private JPanel buildMessagePanel()
	{
		JPanel panel = new JPanel(new BorderLayout());

		panel.add(buildSourceAndInformationPanel(), BorderLayout.NORTH);
		panel.add(buildFileActionsAndViewPanel(), BorderLayout.CENTER);

		return panel;
	}


	/**
	 * Build a panel containing the source select list, the selected
	 * file list, and the current file information sub-panels.
	 * These three panels are side by side with the information panel
	 * taking any extra space.
	 */
	private JPanel buildSourceAndInformationPanel()
	{
		JPanel panel2 = new JPanel(new BorderLayout());

		panel2.add(buildFileSelectionPanel(), BorderLayout.WEST);
		panel2.add(buildInformationPanel(), BorderLayout.CENTER);

		JPanel panel = new JPanel(new BorderLayout());

		panel.add(buildSourcePanel(), BorderLayout.WEST);
		panel.add(panel2, BorderLayout.CENTER);

		return panel;
	}


	/**
	 * Build the select list panel to choose which files are
	 * being processed by this dialog, and a button which performs
	 * the selection process to allow the rest of the dialog to
	 * use the new data.
	 */
	private JPanel buildSourcePanel()
	{
		JPanel panel = new JPanel(new BorderLayout());

		panel.setBorder(BorderFactory.createTitledBorder("Select List"));

		//
		// Build the full select list list.
		//
		mSelectListList = Utils.buildSelectListList(mManager, true);

		//
		// For convenience, select those select lists in the select
		// list list which were selected in the select list table.
		//
		Utils.setSelectedSelectLists(mSelectListTable, mSelectListList);

		//
		// Wrap the list in a scroll pane and add it to the panel.
		//
		panel.add(new JScrollPane(mSelectListList), BorderLayout.CENTER);

		//
		// Allocate the button to apply the select list in its own
		// flowlayout panel and then add that to the source panel.
		//
		mApplySelectListButton = new JButton("Use List");

		mApplySelectListButton.addActionListener(this);

		JPanel buttonPanel = new JPanel();

		buttonPanel.add(mApplySelectListButton);

		panel.add(buttonPanel, BorderLayout.SOUTH);

		return panel;
	}


	/**
	 * Build the file selection panel.
	 * This has a JList which shows the list of files being examined
	 * for the selected select list and buttons to move through the list.
	 */
	private JPanel buildFileSelectionPanel()
	{
		//
		// Create the panel to hold the file list.
		//
		mFileListPanel = new JPanel(new BorderLayout());

		//
		// Create titled borders for the panel.
		// The second one is a dummy border used to get around a swing bug.
		//
		mFileListBorder = BorderFactory.createTitledBorder("File List");
		mDummyBorder = BorderFactory.createTitledBorder("File List");

		//
		// Set the title border for the panel.
		//
		mFileListPanel.setBorder(mFileListBorder);

		//
		// Build a JList with autoscroll and with only one row selectable.
		//
		mFileListList = new JList<FileInfo>();

		mFileListList.setAutoscrolls(true);
		mFileListList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		//
		// Set the size of the viewport to a useful value.
		//
		mFileListList.setVisibleRowCount(Utils.getListVisibleRowCount(10));

		//
		// Make ourself the listener for selections and keys.
		//
		mFileListList.addListSelectionListener(this);
		mFileListList.addKeyListener(this);

		//
		// Wrap the list in a scroll pane and add it to the panel.
		//
		mFileListScrollPane = new JScrollPane(mFileListList);

		mFileListPanel.add(mFileListScrollPane, BorderLayout.CENTER);

		//
		// Add the actions buttons to the bottom.
		//
		mFileListPanel.add(buildInformationActionsPanel(), BorderLayout.SOUTH);

		return mFileListPanel;
	}


	/**
	 * Build the file information panel.
	 * This contains a summary of the number of available in the
	 * select list, the currently selected file position,
	 * information about the currently selected file, and buttons
	 * to move through the selected files.
	 */
	private JPanel buildInformationPanel()
	{
		JPanel panel = new JPanel(new BorderLayout());

		panel.setBorder(BorderFactory.createTitledBorder(
			"Current File Information"));

		//
		// Build vertical panels to hold the labels and text fields.
		//
		JPanel labelPanel = new JPanel(new GridLayout(0, 1));
		JPanel statusPanel = new JPanel(new GridLayout(0, 1));

		//
		// Add the file information text fields.
		//
		mFullPathTextField = addInformationField(labelPanel, statusPanel, "Full Path: ");
		mFileNameTextField = addInformationField(labelPanel, statusPanel, "File Name: ");
		mFileDepthTextField = addInformationField(labelPanel, statusPanel, "Depth: ");
		mFileKeyTextField = addInformationField(labelPanel, statusPanel, "Key: ");
		mFileTypeTextField = addInformationField(labelPanel, statusPanel, "Type: ");
		mFileContentTypeTextField = addInformationField(labelPanel, statusPanel, "Content Type: ");
		mFileLengthTextField = addInformationField(labelPanel, statusPanel, "Length: ");
		mFileOwnerTextField = addInformationField(labelPanel, statusPanel, "Owner: ");
		mFileGroupTextField = addInformationField(labelPanel, statusPanel, "Group: ");
		mFileCreationTextField = addInformationField(labelPanel, statusPanel, "Creation Time: ");
		mFileLastModifiedTextField = addInformationField(labelPanel, statusPanel, "Modified Time: ");
		mFileLastAccessTextField = addInformationField(labelPanel, statusPanel, "Access Time: ");
		mFileLinkValueTextField = addInformationField(labelPanel, statusPanel, "Link Value: ");
		mFileChecksumTextField = addInformationField(labelPanel, statusPanel, "Checksum: ");
		mFilePermissionsTextField = addInformationField(labelPanel, statusPanel, "Permissions: ");
		mFileAccessTextField = addInformationField(labelPanel, statusPanel, "Access: ");
		mFileSelectListsField = addInformationField(labelPanel, statusPanel, "Select Lists: ");

		//
		// Put the two panels side by side in a panel.
		//
		JPanel infoPanel = new JPanel(new BorderLayout());

		infoPanel.add(labelPanel, BorderLayout.WEST);
		infoPanel.add(statusPanel, BorderLayout.CENTER);

		//
		// Add the information panel to the final panel with a
		// blank component under it to consume the space under
		// the information.
		//
		panel.add(infoPanel, BorderLayout.NORTH);
		panel.add(new JPanel(), BorderLayout.CENTER);

		return panel;
	}


	/**
	 * Add a label and text field to display a piece of file information
	 * into the specified panels.  Returns the created text field.
	 */
	private JTextField addInformationField(JPanel labelPanel, JPanel statusPanel, String label)
	{
		labelPanel.add(new JLabel(label, SwingConstants.RIGHT));

		JTextField textField = new JTextField(50);

		textField.setEditable(false);

		statusPanel.add(textField);

		return textField;
	}

	/**
	 * Build the information actions panel.
	 * This contains a text field used to search for file names and
	 * buttons to perform forwards and backwards file name searches.
	 */
	private JPanel buildInformationActionsPanel()
	{
		JPanel panel = new JPanel();

		//
		// Allocate a text field to use for file name searches.
		//
		mFindFileNameTextField = new JTextField(10);

		//
		// Allocate the NEXT and PREVIOUS buttons for next searches.
		//
		mFindNextFileButton = new JButton("Next");
		mFindPreviousFileButton = new JButton("Previous");

		//
		// Add ourself as a listener to the text field and buttons.
		//
		mFindFileNameTextField.addActionListener(this);
		mFindNextFileButton.addActionListener(this);
		mFindPreviousFileButton.addActionListener(this);

		//
		// Put the components into the panel and return it.
		//
		panel.add(new JLabel("Find"));
		panel.add(mFindFileNameTextField);
		panel.add(mFindNextFileButton);
		panel.add(mFindPreviousFileButton);

		return panel;
	}


	/**
	 * Build the file actions and view panel.
	 * This contains actions buttons for the current file on the left,
	 * and a text area showing the file contents on the right.
	 * The panel itself is built and run by the ViewFileDialog class.
	 */
	private JPanel buildFileActionsAndViewPanel()
	{
		//
		// Allocate the view file dialog.
		//
		mViewFileDialog = new ViewFileDialog(mMainWindow, mFrame);

		//
		// Make us a listener for the dialog's effects.
		//
		mViewFileDialog.setListener(this);

		//
		// Ask the dialog to build and return the panel which we use.
		//
		return mViewFileDialog.buildViewFilePanel();
	}


	/**
	 * Build the options panel.
	 * This only has the close button.
	 */
	private JPanel buildOptionsPanel()
	{
		JPanel panel = new JPanel();

		mCloseButton = new JButton("Close");

		mCloseButton.addActionListener(this);

		panel.add(mCloseButton);

		return panel;
	}


	/**
	 * Action called when a key is pressed in our file selection list.
	 * We handle the left and right arrow keys to scroll sideways,
	 * and a few keys to force the viewing of the current file.
	 */
	@Override
	public void keyPressed(KeyEvent e)
	{
		//
		// Get the scrollbar's values.
		//
		JScrollBar scrollBar = mFileListScrollPane.getHorizontalScrollBar();

		int value = scrollBar.getValue();
		int maximumValue = scrollBar.getMaximum();
		int increment = scrollBar.getVisibleAmount() / 4;

		if (increment < 10)
			increment = 10;

		//
		// Switch on the keycode to handle the event.
		//
		switch (e.getKeyCode())
		{
			case KeyEvent.VK_RIGHT:
				//
				// Scroll right by a bit.
				//
				value += increment;

				if (value > maximumValue)
					value = maximumValue;

				break;

			case KeyEvent.VK_LEFT:
				//
				// Scroll left by a bit.
				//
				value -= increment;

				if (value < 0)
					value = 0;

				break;

			case KeyEvent.VK_V:
				//
				// View the current file.
				//
				mViewFileDialog.viewFileAction();

				break;

			case KeyEvent.VK_B:
				//
				// View the current file as binary.
				//
				mViewFileDialog.viewFileAsBinaryAction();

				break;

			case KeyEvent.VK_T:
				//
				// View the current file as text.
				//
				mViewFileDialog.viewFileAsTextAction();

				break;

			default:
				//
				// Ignore this key.
				//
				return;
		}

		//
		// Set the new scrollbar value.
		//
		scrollBar.setValue(value);
	}


	/**
	 * Action called when a key is released in our view window.
	 */
	@Override
	public void keyReleased(KeyEvent e)
	{
	}


	
	/**
	 * Action called when a key is typed in our view window.
	 */
	@Override
	public void keyTyped(KeyEvent e)
	{
	}


	/**
	 * Implementation of ListSelectionListener method called when a new
	 * selection is made for our file list.  We position to the indicated
	 * file.
	 */
	@Override
	public void valueChanged(ListSelectionEvent e)
	{
		if (e.getSource() != mFileListList)
			return;

		//
		// Get the list index which is selected.
		//
		int index = mFileListList.getSelectedIndex();

		//
		// Set the current file to that index.
		//
		setCurrentFile(index);
	}


	/**
	 * ActionListener method called when one of our buttons is pressed.
	 */
	@Override
	public void actionPerformed(ActionEvent e)
	{
		//
		// Switch on the source button to handle it.
		//
		Object source = e.getSource();

		if (source == mCloseButton)
			closeAction();
		else if (source == mApplySelectListButton)
			applySelectListAction();
		else if (source == mFindNextFileButton)
			findNextFileAction();
		else if (source == mFindPreviousFileButton)
			findPreviousFileAction();
		else if (source == mFindFileNameTextField)
			findFirstFileAction();

		//
		// Fire any pending updates that the action might have caused.
		//
		mManager.firePendingRowUpdates();
	}


	/**
	 * Perform the close action.
	 */
	private void closeAction()
	{
		if (mFrame == null)
			return;

		//
		// Save our settings for next time.
		//
		mSettings.setValue(PROP_INTERACTIVE_WINDOW_SIZE,
			mMainPanel.getSize());

		//
		// Tell our view file dialog to save its settings too.
		//
		mViewFileDialog.saveSettings();

		//
		// Close down the frame.
		//
		mFrame.dispose();
		mFrame = null;
	}


	/**
	 * Perform the Apply Select List action.
	 * This populates the list of files in the file JList, positions to the
	 * first file in the list, and updates the current position information.
	 */
	private void applySelectListAction()
	{
		//
		// Get the set of files to be acted on.
		//
		BitSet fileBitSet = Utils.getSelectedFileBitSet(mSelectListList);

		//
		// If there is nothing selected then do nothing.
		//
		if (fileBitSet == null)
			return;

		//
		// Allocate an array to hold the list of selected files.
		//
		mFileArraySize = fileBitSet.cardinality();

		mFileArray = new FileInfo[mFileArraySize];

		//
		// Put the selected files into the array.
		//
		int arrayIndex = 0;
		int fileIndex = -1;

		while (true)
		{
			fileIndex = fileBitSet.nextSetBit(fileIndex + 1);

			if (fileIndex < 0)
				break;

			mFileArray[arrayIndex++] = mManager.getFileInfo(fileIndex);
		}

		//
		// Set the list's data to the array of files.
		//
		mFileListList.setListData(mFileArray);

		//
		// Position ourself on the first file.
		//
		setCurrentFile(0);
	}


	/**
	 * Find the first file whose name contains the find file name text.
	 * If there is no such file then the current file is cleared.
	 */
	private void findFirstFileAction()
	{
		//
		// Get the text to be found.
		//
		String text = mFindFileNameTextField.getText();

		//
		// Search the file names looking for a match.
		//
		for (int index = 0; index < mFileArraySize; index++)
		{
			if (isFileNameMatched(index, text))
			{
				setCurrentFile(index);

				return;
			}
		}

		//
		// There was no match so clear the current file.
		//
		setCurrentFile(-1);
	}


	/**
	 * Find the next file whose name contains the find file name text.
	 * If there is no such file then the current file is cleared.
	 */
	private void findNextFileAction()
	{
		//
		// Get the text to be found.
		//
		String text = mFindFileNameTextField.getText();

		//
		// Get the beginning index to search depending on whether or not
		// there is a current file.
		//
		int index = ((mCurrentFile != null) ? mFileArrayIndex : -1);

		//
		// Search the next files looking for a match.
		//
		while (++index < mFileArraySize)
		{
			if (isFileNameMatched(index, text))
			{
				setCurrentFile(index);

				return;
			}
		}

		//
		// There was no match so clear the current file.
		//
		setCurrentFile(-1);
	}


	/**
	 * Find the previous file whose name contains the find file name text.
	 */
	private void findPreviousFileAction()
	{
		//
		// Get the text to be found.
		//
		String text = mFindFileNameTextField.getText();

		//
		// Get the beginning index to search depending on whether or not
		// there is a current file.
		//
		int index = ((mCurrentFile != null) ? mFileArrayIndex : mFileArraySize);

		//
		// Search the previous files looking for a match.
		//
		while (--index >= 0)
		{
			if (isFileNameMatched(index, text))
			{
				setCurrentFile(index);

				return;
			}
		}

		//
		// There was no match so clear the current file.
		//
		setCurrentFile(-1);
	}


	/**
	 * Return whether or not the file name at the specified index in the
	 * file array has a name which is matched by the indicated text.
	 * The match is against the last component of the file name path.
	 * Matching means that the text is contained somewhere within the name.
	 * An out of range index returns false.
	 */
	private boolean isFileNameMatched(int index, String text)
	{
		if ((index < 0) || (index >= mFileArraySize))
			return false;

		if ((text == null) || (text.length() == 0))
			return true;

		String fileName = mFileArray[index].getLastComponent();

		fileName = fileName.toLowerCase();
		text = text.toLowerCase();

		return (fileName.indexOf(text) >= 0);
	}


	/**
	 * Position to the indicated position in the selected list of files
	 * and display the information about it.  An out of range index results
	 * in no file being viewed.
	 */
	public void setCurrentFile(int index)
	{
		//
		// Clear the current file in case the index is bad.
		//
		mCurrentFile = null;
		mFileArrayIndex = -1;

		//
		// If the index is valid then set the index and the current file.
		//
		if ((index >= 0) && (index < mFileArraySize))
		{
			mFileArrayIndex = index;
			mCurrentFile = mFileArray[index];
		}

		//
		// Show the current file.
		//
		showCurrentFile();
	}


	/**
	 * Implementation of ViewFileListener method to show the next file.
	 */
	@Override
	public void viewFileNextFileRequested()
	{
		if ((mFileArrayIndex >= 0) && (mFileArrayIndex + 1 < mFileArraySize))
			setCurrentFile(mFileArrayIndex + 1);
	}


	/**
	 * Implementation of ViewFileListener method to show the previous file.
	 */
	@Override
	public void viewFilePreviousFileRequested()
	{
		if (mFileArrayIndex > 0)
			setCurrentFile(mFileArrayIndex - 1);
	}


	/**
	 * Implementation of ViewFileListener method to update the select list
	 * information field for the current file.
	 */
	@Override
	public void viewFileSelectListChanged()
	{
		//
		// If there is no current file then clear the field.
		//
		if (mCurrentFile == null)
		{
			mFileSelectListsField.setText(null);

			return;
		}

		//
		// Get the index of the file for checking.
		//
		int fileIndex = mCurrentFile.getIndex();

		//
		// Get an array of all of the select lists.
		//
		AbstractSelectList[] selectListArray = mManager.getSelectListArray();

		//
		// Allocate a string buffer to build up the string.
		//
		StringBuilder buffer = new StringBuilder(64);

		//
		// Iterate over all of the select lists seeing if the file is
		// in each one.
		//
		for (AbstractSelectList selectList : selectListArray)
		{
			if (selectList.isFileInList(fileIndex))
			{
				//
				// Append a space to separate the select list names.
				//
				if (buffer.length() > 0)
					buffer.append(' ');

				//
				// Add this select list's name to the buffer.
				//
				buffer.append(selectList.getName());
			}
		}

		//
		// Put the completed value into the text field.
		//
		mFileSelectListsField.setText(buffer.toString());
	}


	/**
	 * Implementation of ViewFileListener method to update the complete
	 * file status for the current file.
	 */
	@Override
	public void viewFileStatusChanged()
	{
		//
		// Blank out the fields if there is no current file.
		//
		if (mCurrentFile == null)
		{
			mFullPathTextField.setText("");
			mFileNameTextField.setText("");
			mFileDepthTextField.setText("");
			mFileKeyTextField.setText("");
			mFileTypeTextField.setText("");
			mFileContentTypeTextField.setText("");
			mFileLengthTextField.setText("");
			mFileOwnerTextField.setText("");
			mFileGroupTextField.setText("");
			mFileCreationTextField.setText("");
			mFileLastModifiedTextField.setText("");
			mFileLastAccessTextField.setText("");
			mFileLinkValueTextField.setText("");
			mFileChecksumTextField.setText("");
			mFilePermissionsTextField.setText("");
			mFileAccessTextField.setText("");
			mFileSelectListsField.setText("");

			return;
		}

		//
		// Set the values from the current file info.
		//
		String fullPath = mCurrentFile.getFileName();
		String fileName = mCurrentFile.getLastComponent();
		String depth = String.valueOf(mCurrentFile.getDepth());
		String key = mCurrentFile.getKey();
		String type = mCurrentFile.getFileType().toString();
		String contentType = mCurrentFile.getContentType();
		String length = String.valueOf(mCurrentFile.getFileLength());
		String owner = mCurrentFile.getOwner();
		String group = mCurrentFile.getGroup();
		String creationTime = TimeValue.format(mCurrentFile.getCreationTime());
		String modifiedTime = TimeValue.format(mCurrentFile.getLastModifiedTime());
		String accessTime = TimeValue.format(mCurrentFile.getLastAccessTime());
		String linkValue = mCurrentFile.getLinkValue();
		String checksum = mCurrentFile.getChecksum();
		String permissions = mCurrentFile.getPermissions();
		String access = buildAccessString(mCurrentFile);

		mFullPathTextField.setText(fullPath);
		mFileNameTextField.setText(fileName);
		mFileDepthTextField.setText(depth);
		mFileKeyTextField.setText(key);
		mFileTypeTextField.setText(type);
		mFileContentTypeTextField.setText(contentType);
		mFileLengthTextField.setText(length);
		mFileOwnerTextField.setText(owner);
		mFileGroupTextField.setText(group);
		mFileCreationTextField.setText(creationTime);
		mFileLastModifiedTextField.setText(modifiedTime);
		mFileLastAccessTextField.setText(accessTime);
		mFileLinkValueTextField.setText(linkValue);
		mFileChecksumTextField.setText(checksum);
		mFilePermissionsTextField.setText(permissions);
		mFileAccessTextField.setText(access);

		//
		// Update the select lists the file is in.
		//
		viewFileSelectListChanged();
	}


	/**
	 * Calculate the file's access as a string (e.g., read/write).
	 */
	private String buildAccessString(FileInfo fileInfo)
	{
		boolean readable = fileInfo.isReadable();
		boolean writable = fileInfo.isWritable();
		boolean executable = fileInfo.isExecutable();

		StringBuilder builder = new StringBuilder();
		int count = 0;

		if (readable)
		{
			builder.append("read");
			count++;
		}

		if (writable)
		{
			if (count++ > 0)
				builder.append('/');

			builder.append("write");
		}

		if (executable)
		{
			if (count++ > 0)
				builder.append('/');

			builder.append("execute");
		}

		if (count == 0)
			builder.append("none");
		else if (count == 1)
			builder.append(" only");

		String access = builder.toString();
	
		return access;
	}


	/**
	 * Update the file list to make the current file selected.
	 */
	private void selectCurrentFile()
	{
		//
		// If there is no file position then clear the selection.
		//
		if (mFileArrayIndex < 0)
		{
			mFileListList.clearSelection();

			return;
		}

		//
		// Select the indicated file and make it visible in the viewport.
		//
		mFileListList.setSelectedIndex(mFileArrayIndex);
		mFileListList.ensureIndexIsVisible(mFileArrayIndex);
	}


	/**
	 * Show the new file position in the title border.
	 */
	private void showFilePositionInBorder()
	{
		//
		// Build the title indicating the file position.
		//
		String title = "File List";

		if (mCurrentFile != null)
		{
			title += " ( selected file " + (mFileArrayIndex + 1) + " of " +
				mFileArraySize + " )";
		}

		//
		// Update the border title of the file list panel.
		// NOTE: Due to a swing bug, updating the border tile directly does
		// not cause the new border to be seen until a refresh is performed.
		// To get around this, we temporarily set another border so that
		// when we set the changed one, it will be property drawn.
		//
		mFileListPanel.setBorder(mDummyBorder);
		mFileListBorder.setTitle(title);
		mFileListPanel.setBorder(mFileListBorder);
	}


	/**
	 * Show the complete information for a new current file.
	 */
	private void showCurrentFile()
	{
		//
		// Show the file position in the title border.
		//
		showFilePositionInBorder();

		//
		// Select the current file.
		//
		selectCurrentFile();

		//
		// Update the status information.
		//
		viewFileStatusChanged();

		//
		// Update the view of the file.
		//
		mViewFileDialog.setFileInfo(mCurrentFile);
	}
}
