/*
 * File Selection - A java utility to browse and manipulate sets of files.
 * Copyright (C) 2013  David I. Bell
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fileselection.dialog;

import java.awt.Component;
import java.awt.BorderLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JOptionPane;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JButton;

import fileselection.FileSelectionManager;
import fileselection.MainWindow;
import fileselection.ManualSelectList;

import fileselection.utilities.Utils;


/**
 * Class to build and run a dialog to load the file names
 * from a file into a manual select list.
 */
public class LoadSelectListFileNamesDialog implements ActionListener
{
	/**
	 * The file name text field.
	 */
	private JTextField mFileNameTextField;

	/**
	 * The label for the file name field.
	 */
	private JLabel mFileNameLabel;

	/**
	 * The button to bring up a file chooser.
	 */
	private JButton mFileChooserButton;

	/**
	 * The select list JList.
	 */
	private JList<ManualSelectList> mSelectListList;

	/**
	 * The manual SelectList which is returned.
	 */
	private ManualSelectList mSelectList;

	/**
	 * The load file name.
	 */
	private String mLoadFileName;

	/**
	 * The file selection manager.
	 */
	private final FileSelectionManager mManager;

	/**
	 * Our parent component.
	 */
	private final Component mParent;

	/**
	 * The select list table.
	 */
	private final JTable mSelectListTable;


	/**
	 * Constructor which sets the main window.
	 */
	public LoadSelectListFileNamesDialog(MainWindow mainWindow)
	{
		//
		// Extract the values we need.
		//
		mManager = mainWindow.getManager();
		mParent = mainWindow.getJFrame();
		mSelectListTable = mainWindow.getSelectListJTable();
	}


	/**
	 * Show the Load File Names from File Dialog displaying a list
	 * of select lists that may be chosen from.  Returns true if the
	 * user used the OK button.  After this the selected SelectList
	 * and load filename can be obtained.
	 */
	public boolean show()
	{
		//
		// Allocate the panel holding the message.
		//
		JPanel mainPanel = new JPanel(new BorderLayout());

		//
		// Build and add the sub-panels.
		//
		mainPanel.add(buildSelectListPanel(), BorderLayout.CENTER);
		mainPanel.add(buildFileNamePanel(), BorderLayout.SOUTH);

		//
		// Show the dialog and get which button was used.
		//
		int result = JOptionPane.showConfirmDialog(mParent,
			mainPanel, "Load File Names into Select List",
			JOptionPane.OK_CANCEL_OPTION);

		//
		// If the dialog was cancelled then return false.
		//
		if (result != JOptionPane.OK_OPTION)
			return false;

		//
		// Save the select list and file name.
		//
		mSelectList = (ManualSelectList)
			mSelectListList.getSelectedValue();

		mLoadFileName = Utils.trimSpaces(mFileNameTextField.getText());

		return true;
	}


	/**
	 * Build the select list panel.
	 */
	private JPanel buildSelectListPanel()
	{
		JPanel panel = new JPanel(new BorderLayout());

		//
		// Build the manual select list list.
		//
		mSelectListList = Utils.buildManualSelectListList(
			mManager, false, true);

		//
		// Wrap the list in a scroll pane and add it to the panel.
		//
		JScrollPane scrollPane = new JScrollPane(mSelectListList);

		panel.add(scrollPane, BorderLayout.CENTER);

		//
		// For convenience, select those select lists in the select
		// list list which were selected in the select list table.
		//
		Utils.setSelectedSelectLists(mSelectListTable, mSelectListList);

		return panel;
	}


	/**
	 * Build the panel containing the file name to load the file
	 * names from.  This has a label, a file name field, and a
	 * choose button which brings up a file chooser dialog.
	 */
	private JPanel buildFileNamePanel()
	{
		mFileNameLabel = new JLabel("File Name");
		mFileChooserButton = new JButton("Find...");
		mFileNameTextField = new JTextField(30);

		//
		// Have the button call us when used.
		//
		mFileChooserButton.addActionListener(this);

		JPanel panel = new JPanel(new BorderLayout());

		panel.add(mFileNameLabel, BorderLayout.WEST);
		panel.add(mFileNameTextField, BorderLayout.CENTER);
		panel.add(mFileChooserButton, BorderLayout.EAST);

		return panel;
	}


	/**
	 * Method called when our choose button is used.
	 * We bring up a file chooser.
	 */
	@Override
	public void actionPerformed(ActionEvent e)
	{
		//
		// Prompt the user for a file.
		//
		JFileChooser chooser = new JFileChooser();

		chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		chooser.setMultiSelectionEnabled(false);
		chooser.setAcceptAllFileFilterUsed(true);
		chooser.setDialogTitle("File to Load File Names From");

		int result = chooser.showOpenDialog(mParent);

		//
		// If the user cancelled then do nothing.
		//
		if (result != JFileChooser.APPROVE_OPTION)
			return;

		//
		// Get the selected file.
		//
		File file = chooser.getSelectedFile();

		//
		// Set the text field value to the file's path.
		//
		mFileNameTextField.setText(file.getAbsolutePath());
	}


	/**
	 * Return the ManualSelectList to be loaded.
	 */
	public ManualSelectList getSelectList()
	{
		return mSelectList;
	}


	/**
	 * Return the load file name.
	 */
	public String getLoadFileName()
	{
		return mLoadFileName;
	}
}
