/*
 * File Selection - A java utility to browse and manipulate sets of files.
 * Copyright (C) 2013  David I. Bell
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fileselection.dialog;

import javax.swing.JOptionPane;

import fileselection.MainWindow;

import fileselection.operation.UpdateFileStatusOperation;


/**
 * Class to run the update file status operation dialog.
 * Actually, since there are no parameters for this operation we have
 * no dialog to show except for error messages and the final results.
 */
public class UpdateFileStatusDialog extends AbstractOperationDialog
{
	/**
	 * The actual update file status operation.
	 */
	private UpdateFileStatusOperation mOperation;


	/**
	 * Constructor which sets the main window.
	 */
	public UpdateFileStatusDialog(MainWindow mainWindow)
	{
		super(mainWindow);
	}


	/**
	 * Run update file status "dialog" by running the add files
	 * operation class, showing progress through a progress dialog.
	 */
	@Override
	public void runDialog()
	{
		//
		// Allocate the update file status operation class.
		//
		mOperation = new UpdateFileStatusOperation(mManager);

		//
		// Check the parameters and if they are wrong then display
		// an error message.  Since we have no parameters this call
		// should never fail.
		//
		if (!mOperation.isReadyForExecution())
		{
			showWarning(mOperation.getErrorMessage());

			return;
		}

		//
		// Everything is ready.
		// Create the progress monitor dialog and inform the
		// update file status operation class about it.
		//
		ProgressMonitorDialog progress =
			new ProgressMonitorDialog("Update File Status Progress", this);

		mOperation.setListener(progress);

		//
		// Do the actual update file status operation.
		// This operation does not block since it is executed in a
		// background thread.
		//
		if (!mOperation.startOperation())
			showWarning("Failed to execute operation");
	}


	/**
	 * Called by the progress listener when the operation has
	 * completed to show the results in the Swing thread.
	 */
	@Override
	public void showResults()
	{
		//
		// Update the file list before showing the results.
		//
		mManager.firePendingRowUpdates();

		//
		// Say how many files of each status we processed.
		//
		String message =
			"Files Processed: " + mOperation.getProcessedFileCount() +
			", Files Updated: " + mOperation.getUpdatedFileCount();

		JOptionPane.showMessageDialog(mParent, message,
			"Update File Status Complete", JOptionPane.INFORMATION_MESSAGE);
	}
}
