/*
 * File Selection - A java utility to browse and manipulate sets of files.
 * Copyright (C) 2013  David I. Bell
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fileselection.operation;

import java.nio.file.Path;

import java.util.BitSet;

import fileselection.FileInfo;
import fileselection.FileSelectionManager;
import fileselection.ManualSelectList;
import fileselection.utilities.Utils;


/**
 * The probe content type operation for the files in the file table.
 * Select lists can be set to indicate which files succeeded or failed.
 */
public class ProbeContentTypeOperation extends AbstractOperation
{
	/**
	 * The bitset which indicates which files are to be probed.
	 */
	private BitSet mFileBitSet;

	/**
	 * The manual select list which gets the files which were successful.
	 * This can be null if this result is not desired.
	 */
	private ManualSelectList mSuccessSelectList;

	/**
	 * The manual select list which gets the files which fail.
	 * This can be null if this result is not desired.
	 */
	private ManualSelectList mFailureSelectList;

	/**
	 * The new bit set for the success select list.
	 */
	private BitSet mSuccessBitSet;

	/**
	 * The new bit set for the failure select list.
	 */
	private BitSet mFailureBitSet;

	/**
	 * Whether we clear or append to the success select list.
	 */
	private boolean mIsSuccessSelectListCleared;

	/**
	 * Whether we clear or append to the failure select list.
	 */
	private boolean mIsFailureSelectListCleared;


	/**
	 * Constructor which sets our file selection manager.
	 */
	public ProbeContentTypeOperation(FileSelectionManager manager)
	{
		super(manager);
	}


	/**
	 * Set the bit set which specifies the files to probe.
	 */
	public void setProbeFileBitSet(BitSet fileBitSet)
	{
		mFileBitSet = fileBitSet;
	}


	/**
	 * Set the manual select list which gets the files which succeeded.
	 * This can be null if this result is not desired.
	 */
	public void setSuccessSelectList(ManualSelectList selectList)
	{
		mSuccessSelectList = selectList;
	}


	/**
	 * Set the manual select list which gets the files which fail.
	 * This can be null if this result is not desired.
	 */
	public void setFailureSelectList(ManualSelectList selectList)
	{
		mFailureSelectList = selectList;
	}


	/**
	 * Whether we clear or append to the success select list.
	 */
	public void setIsSuccessSelectListCleared(boolean isCleared)
	{
		mIsSuccessSelectListCleared = isCleared;
	}


	/**
	 * Whether we clear or append to the failure select list.
	 */
	public void setIsFailureSelectListCleared(boolean isCleared)
	{
		mIsFailureSelectListCleared = isCleared;
	}


	/**
	 * Return the count of files which were successful.
	 */
	public int getSuccessFileCount()
	{
		return mSuccessBitSet.cardinality();
	}


	/**
	 * Return the count of files which we failed to probe.
	 */
	public int getFailureFileCount()
	{
		return mFailureBitSet.cardinality();
	}


	/**
	 * Check that the parameters have been properly set for the
	 * operation to begin, and if not save an error message which
	 * can be obtained using getParameterErrorMessage.
	 */
	@Override
	public boolean isReadyForExecution()
	{
		//
		// We must have a file bit set.
		//
		if (mFileBitSet == null)
			return setErrorAndReturnFalse("No probe content type select list set");

		//
		// We are ready for the probing.
		// Say that we are ready.
		//
		setIsReadyForExecution();

		return true;
	}


	/**
	 * Run this operation.  This performs the probe operation on the files set
	 * by the previously prepared arguments.  The results are stored in
	 * the FileInfo objects for future reference.
	 */
	@Override
	public void runOperation()
	{
		//
		// Get the total count of files to probe.
		//
		int fileCount = mManager.getFileCount();
		int processedCount = 0;
		int failedCount = 0;

		//
		// Tell the listener the number of files.
		//
		updateTotalFileCount(fileCount);

		//
		// Allocate new bit sets to hold the results.
		//
		mSuccessBitSet = new BitSet(fileCount);
		mFailureBitSet = new BitSet(fileCount);

		//
		// Iterate over all of the files in the file table.
		//
		for (int index = 0; index < fileCount; index++)
		{
			//
			// If this file is not in the bit set then continue.
			//
			if (!mFileBitSet.get(index))
				continue;

			//
			// Get the file handle for this index and let the
			// listener know about it.
			//
			FileInfo fileInfo = mManager.getFileInfo(index);

			updateFileName(fileInfo.getFileName());

			//
			// Perform the probe and get the status.
			//
			boolean successFlag = probeContentType(fileInfo);

			//
			// If we are being cancelled then abort the operation now.
			//
			if (isCancelled())
				return;

			//
			// Update the status.
			//
			updateProcessedFileCount(++processedCount);

			if (successFlag)
				mSuccessBitSet.set(index);
			else
			{
				mFailureBitSet.set(index);
				updateFailedFileCount(++failedCount);
			}
		}

		//
		// The results are done.
		// Assign or update the select list bit sets if necessary.
		//
		if (mSuccessSelectList != null)
		{
			if (mIsSuccessSelectListCleared)
				mSuccessSelectList.setFileBitSet(mSuccessBitSet);
			else
				mSuccessSelectList.orOperation(mSuccessBitSet);
		}

		if (mFailureSelectList != null)
		{
			if (mIsFailureSelectListCleared)
				mFailureSelectList.setFileBitSet(mFailureBitSet);
			else
				mFailureSelectList.orOperation(mFailureBitSet);
		}
	}


	/**
	 * Perform the content type probe on the indicated file or directory.
	 * The result is stored in the FileInfo for later reference.
	 * Returns true if the probing was successful.
	 */
	private boolean probeContentType(FileInfo fileInfo)
	{
		//
		// Get a file handle for the file.
		//
		Path path = fileInfo.getPath();

		//
		// Try to determine the content type of the file.
		//
		String contentType = Utils.probeContentType(path);

		//
		// Set the content type into the file if we haven't been cancelled.
		//
		if (!isCancelled())
			fileInfo.setContentType(contentType);

		//
		// Return the probe status.
		//
		return (contentType != null);
	}
}
