/*
 * File Selection - A java utility to browse and manipulate sets of files.
 * Copyright (C) 2013  David I. Bell
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fileselection.operation;

import java.io.InputStream;
import java.io.BufferedInputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;

import java.nio.file.Path;
import java.nio.file.Files;

import java.util.BitSet;

import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import fileselection.FileInfo;
import fileselection.FileSelectionManager;
import fileselection.ManualSelectList;

import fileselection.utilities.Utils;


/**
 * The search operation for the files in the file table.
 * This reads all of the files in a specified select list looking
 * for lines which match the specified pattern.  Files which have a
 * line matching the pattern are added to a specified select list,
 * whereas files without a matching line are added to another select
 * list.  If a file could not be read it is added to a specified
 * failure select list instead.  The destination select lists can
 * be the search select list with no problems.
 */
public class SearchOperation extends AbstractOperation
{
	/**
	 * The bitset which indicates which files are to be searched.
	 */
	private BitSet mFileBitSet;

	/**
	 * The manual select list which gets the files with found matches.
	 * This can be null if this result is not desired.
	 */
	private ManualSelectList mMatchingSelectList;

	/**
	 * The manual select list which gets the files without matches.
	 * This can be null if this result is not desired.
	 */
	private ManualSelectList mNotMatchingSelectList;

	/**
	 * The manual select list which gets the files which fail.
	 * This can be null if this result is not desired.
	 */
	private ManualSelectList mFailureSelectList;

	/**
	 * The new bit set for the matching select list.
	 */
	private BitSet mMatchingBitSet;

	/**
	 * The new bit set for the not matching select list.
	 */
	private BitSet mNotMatchingBitSet;

	/**
	 * The new bit set for the failure select list.
	 */
	private BitSet mFailureBitSet;

	/**
	 * The search pattern string to look for in the files.
	 */
	private String mSearchString;

	/**
	 * Whether case matters in the search.
	 */
	private boolean mIsCaseSensitive;

	/**
	 * Whether regular expressions are used.
	 */
	private boolean mIsRegularExpression;

	/**
	 * Whether we clear or append to the matching select list.
	 */
	private boolean mIsMatchingSelectListCleared;

	/**
	 * Whether we clear or append to the not matching select list.
	 */
	private boolean mIsNotMatchingSelectListCleared;

	/**
	 * Whether we clear or append to the failure select list.
	 */
	private boolean mIsFailureSelectListCleared;

	/**
	 * The compiled search pattern.
	 */
	private Pattern mSearchPattern;

	/**
	 * The matcher for a pattern search.
	 */
	private Matcher mMatcher;


	/**
	 * Constructor which sets our file selection manager.
	 */
	public SearchOperation(FileSelectionManager manager)
	{
		super(manager);
	}


	/**
	 * Set the bit set which specifies the files to search.
	 */
	public void setSearchFileBitSet(BitSet fileBitSet)
	{
		mFileBitSet = fileBitSet;
	}


	/**
	 * Set the manual select list which gets the files with found matches.
	 * This can be null if this result is not desired.
	 */
	public void setMatchingSelectList(ManualSelectList selectList)
	{
		mMatchingSelectList = selectList;
	}


	/**
	 * Set the manual select list which gets the files without matches.
	 * This can be null if this result is not desired.
	 */
	public void setNotMatchingSelectList(ManualSelectList selectList)
	{
		mNotMatchingSelectList = selectList;
	}


	/**
	 * Set the manual select list which gets the files which fail.
	 * This can be null if this result is not desired.
	 */
	public void setFailureSelectList(ManualSelectList selectList)
	{
		mFailureSelectList = selectList;
	}


	/**
	 * Set the search pattern string to look for in the files.
	 */
	public void setSearchPattern(String searchString)
	{
		mSearchString = searchString;
	}


	/**
	 * Set whether case matters in the search.
	 */
	public void setIsCaseSensitive(boolean isCaseSensitive)
	{
		mIsCaseSensitive = isCaseSensitive;
	}


	/**
	 * Set whether regular expressions are used.
	 */
	public void setIsRegularExpression(boolean isRegularExpression)
	{
		mIsRegularExpression = isRegularExpression;
	}


	/**
	 * Whether we clear or append to the matching select list.
	 */
	public void setIsMatchingSelectListCleared(boolean isCleared)
	{
		mIsMatchingSelectListCleared = isCleared;
	}


	/**
	 * Whether we clear or append to the not matching select list.
	 */
	public void setIsNotMatchingSelectListCleared(boolean isCleared)
	{
		mIsNotMatchingSelectListCleared = isCleared;
	}


	/**
	 * Whether we clear or append to the failure select list.
	 */
	public void setIsFailureSelectListCleared(boolean isCleared)
	{
		mIsFailureSelectListCleared = isCleared;
	}


	/**
	 * Return the count of files which had a match.
	 */
	public int getMatchingFileCount()
	{
		return mMatchingBitSet.cardinality();
	}


	/**
	 * Return the count of files which did not have a match.
	 */
	public int getNotMatchingFileCount()
	{
		return mNotMatchingBitSet.cardinality();
	}


	/**
	 * Return the count of files which we failed to search.
	 */
	public int getFailureFileCount()
	{
		return mFailureBitSet.cardinality();
	}


	/**
	 * Check that the parameters have been properly set for the
	 * operation to begin, and if not save an error message which
	 * can be obtained using getParameterErrorMessage.
	 */
	@Override
	public boolean isReadyForExecution()
	{
		//
		// Clear the compiled pattern.
		//
		mSearchPattern = null;
		mMatcher = null;

		//
		// We must have a search string and file bit set.
		//
		if (mSearchString == null)
			return setErrorAndReturnFalse("No search string set");

		if (mFileBitSet == null)
			return setErrorAndReturnFalse("No search select list set");

		//
		// We must have an output select list.
		//
		if ((mMatchingSelectList == null) &&
			(mNotMatchingSelectList == null) &&
			(mFailureSelectList == null))
		{
			return setErrorAndReturnFalse("No output select list set");
		}

		//
		// If we are doing a pattern search then compile it.
		//
		if (mIsRegularExpression)
		{
			int flags = (mIsCaseSensitive ? 0 : Pattern.CASE_INSENSITIVE);

			//
			// Compile the pattern, catching exceptions.
			//
			try
			{
				mSearchPattern = Pattern.compile(mSearchString, flags);
				mMatcher = mSearchPattern.matcher("");
			}
			catch (PatternSyntaxException e)
			{
				return setErrorAndReturnFalse(
					"Bad regular expression: " + e.getMessage());
			}
		}

		//
		// If the search is not case sensitive then lower case it.
		//
		if (!mIsCaseSensitive)
		{
			//
			// This is a normal string search.
			// If we are being case insensitive then lower case the
			// search string.
			//
			mSearchString = mSearchString.toLowerCase();
		}

		//
		// We are ready for the search.
		// Say that we are ready.
		//
		setIsReadyForExecution();

		return true;
	}


	/**
	 * Run this operation.  This performs the search on the
	 * previously prepared arguments
	 */
	@Override
	public void runOperation()
	{
		//
		// Get the total count of files to search.
		//
		int fileCount = mManager.getFileCount();
		int processedCount = 0;
		int failedCount = 0;

		//
		// Tell the listener the number of files.
		//
		updateTotalFileCount(fileCount);

		//
		// Allocate new bit sets to hold the results.
		//
		mMatchingBitSet = new BitSet(fileCount);
		mNotMatchingBitSet = new BitSet(fileCount);
		mFailureBitSet = new BitSet(fileCount);

		//
		// Iterate over all of the files in the file table.
		//
		for (int index = 0; index < fileCount; index++)
		{
			//
			// If this file is not in the bit set then continue.
			//
			if (!mFileBitSet.get(index))
				continue;

			//
			// Get the file handle for this index and let the
			// listener know about it.
			//
			FileInfo fileInfo = mManager.getFileInfo(index);

			updateFileName(fileInfo.getFileName());


			//
			// Perform the search and get the status.
			//
			Path path = fileInfo.getPath();

			int status = search(path);

			//
			// If we are being cancelled then abort the operation now.
			//
			if (isCancelled())
				return;

			//
			// If the status is positive, then the file matched.
			// If the status is zero, the file is not matched.
			// If the status is negative, the file could not be read.
			//
			updateProcessedFileCount(++processedCount);

			if (status > 0)
				mMatchingBitSet.set(index);
			else if (status == 0)
				mNotMatchingBitSet.set(index);
			else
			{
				mFailureBitSet.set(index);
				updateFailedFileCount(++failedCount);
			}
		}

		//
		// The results are done.
		// Assign or update the select list bit sets if necessary.
		//
		if (mMatchingSelectList != null)
		{
			if (mIsMatchingSelectListCleared)
				mMatchingSelectList.setFileBitSet(mMatchingBitSet);
			else
				mMatchingSelectList.orOperation(mMatchingBitSet);
		}

		if (mNotMatchingSelectList != null)
		{
			if (mIsNotMatchingSelectListCleared)
				mNotMatchingSelectList.setFileBitSet(mNotMatchingBitSet);
			else
				mNotMatchingSelectList.orOperation(mNotMatchingBitSet);
		}

		if (mFailureSelectList != null)
		{
			if (mIsFailureSelectListCleared)
				mFailureSelectList.setFileBitSet(mFailureBitSet);
			else
				mFailureSelectList.orOperation(mFailureBitSet);
		}
	}


	/**
	 * Perform a search for the pattern for the indicated file.
	 * Returns a status value indicating the result of the search.
	 * If the status is positive, then the file matched.
	 * If the status is zero, the file is not matched.
	 * If the status is negative, the file could not be read.
	 */
	private int search(Path path)
	{
		InputStream inputStream = null;
		int status = 0;

		//
		// Catch exceptions so we can close the file and also so
		// that we can report the failure status.
		//
		try
		{
			//
			// Try to open the file for reading.
			//
			inputStream = Files.newInputStream(path);

			//
			// Build a buffered reader from the input stream.
			//
			BufferedInputStream bufferedInputStream =
				new BufferedInputStream(inputStream);

			InputStreamReader inputStreamReader =
				new InputStreamReader(bufferedInputStream);

			BufferedReader reader = new BufferedReader(inputStreamReader);

			//
			// Loop over all of the lines in the file until either
			// we get to the end or until we find a match.
			// Stop early if we are cancelled.
			//
			while (!isCancelled())
			{
				//
				// Read the next line.
				//
				String line = reader.readLine();

				//
				// On end of file break out.
				//
				if (line == null)
					break;

				//
				// If a match is found on the line the set the status to
				// success and break out of the loop.
				//
				if (matches(line))
				{
					status = 1;

					break;
				}
			}

			//
			// Close the file.
			//
			inputStream.close();
			inputStream = null;
		}
		catch (Exception e)
		{
			//
			// Set the status to indicate failure.
			//
			status = -1;
		}
		finally
		{
			//
			// Safely close the file.
			//
			Utils.safeClose(inputStream);
		}

		//
		// Return the search status.
		//
		return status;
	}


	/**
	 * Return whether or not there is a match on the indicated line.
	 */
	private boolean matches(String line)
	{
		//
		// If we are doing regular expressions then ask the matcher
		// whether this line is matched.
		//
		if (mIsRegularExpression)
		{
			mMatcher.reset(line);

			return mMatcher.find();
		}

		//
		// This is a normal string search.
		// Lower case the line if we are being case insensitive.
		//
		if (!mIsCaseSensitive)
			line = line.toLowerCase();

		//
		// Look for the search string within the line.
		//
		return (line.indexOf(mSearchString) >= 0);
	}
}
