/*
 * File Selection - A java utility to browse and manipulate sets of files.
 * Copyright (C) 2013  David I. Bell
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fileselection.utilities;

import java.io.InputStream;
import java.io.OutputStream;

import java.util.ArrayList;


/**
 * A class to run an external program and collect its exit status.
 * Argument quoting and filename replacement capability is provided. 
 * This supplies an immediate end of file to stdin and consumes all
 * output from stdout and stderr.
 */
public class ProgramRunner
{
	/**
	 * The property for the default replacement string.
	 */
	private static final String PROP_DEFAULT_REPLACEMENT_STRING =
		"default.replacement.string";

	/**
	 * The default replacement string for insertion of the file name
	 * into the arguments if there is no settings value.
	 */
	private static final String DEFAULT_REPLACEMENT_STRING = "%";

	/**
	 * The command arguments to be executed.
	 * The first argument is the program name to be executed.
	 */
	private String[] mCommandArguments;

	/**
	 * The replacement string where a file name is to be inserted
	 * into the command arguments.
	 */
	private String mReplacementString;

	/**
	 * The current process we have created.
	 */
	private Process mProcess;

	/**
	 * The message that was set on an error.
	 */
	private String mErrorMessage;

	/**
	 * The exit status from the command.
	 */
	private int mExitStatus;

	/**
	 * Whether the command was forcefully terminated.
	 */
	private boolean mIsTerminated;

	/**
	 * The number of bytes read from the command's stdout.
	 */
	private int mStdoutByteCount;

	/**
	 * The number of bytes read from the command's stderr.
	 */
	private int mStderrByteCount;


	/**
	 * Set the command line arguments to be executed for the program.
	 * This accepts single or double quotes as quoting characters but
	 * otherwise breaks up arguments on spaces.
	 */
	public void setCommandArguments(String line)
	{
		ArrayList<String> list = new ArrayList<String>();
		StringBuilder arg = new StringBuilder();
		int length = line.length();
		int quoteChar = -1;
		boolean haveArg = false;

		//
		// Scan the characters of the line one by one.
		//
		for (int index = 0; index < length; index++)
		{
			//
			// Get the current character.
			//
			int ch = line.charAt(index);

			//
			// If we are looking for a quote character and this is it
			// then clear the quote character and return to normal parsing.
			//
			if (ch == quoteChar)
			{
				quoteChar = -1;

				continue;
			}

			//
			// If this is a quote character then remember it and
			// continue scanning looking for the matching quote.
			//
			if ((ch == '\'') || (ch == '"'))
			{
				quoteChar = ch;
				haveArg = true;

				continue;
			}

			//
			// If the character is not a space or we are inside of a
			// quoted argument then just append the character to the
			// current argument.
			//
			if ((ch != ' ') || (quoteChar >= 0))
			{
				arg.append((char) ch);
				haveArg = true;

				continue;
			}

			//
			// This is a space which breaks up the arguments.
			// If we have seen an argument then store it and reset the
			// state for a new argument.
			//
			if (haveArg)
			{
				list.add(arg.toString());
				arg.setLength(0);
				haveArg = false;
			}
		}

		//
		// Store the final argument if necessary.
		//
		if (haveArg)
			list.add(arg.toString());

		//
		// Build an argument array from the array list.
		//
		String[] commandArray = new String[list.size()];

		list.toArray(commandArray);

		//
		// Set the command arguments using this array.
		//
		setCommandArguments(commandArray);
	}


	/**
	 * Set the command arguments to be executed for the program.
	 * The first argument is the program to be executed.
	 */
	public void setCommandArguments(String[] commandArguments)
	{
		mCommandArguments = commandArguments;
	}


	/**
	 * Set or clear the replacement string which is used to replace text
	 * in the command arguments with the file names.  If this is non-null
	 * then the command arguments are expected to contain this string
	 * somewhere for replacement with a file name.
	 */
	public void setReplacementString(String replacementString)
	{
		mReplacementString = replacementString;
	}


	/**
	 * Set the default replacement string which is used to replace text
	 * in the command arguments with the file names.  This is a property
	 * value or our built in default if the property value is empty.
	 */
	public void setDefaultReplacementString()
	{
		Settings settings = Settings.getInstance();

		mReplacementString = settings.getString(
			PROP_DEFAULT_REPLACEMENT_STRING, "");

		if (mReplacementString.length() == 0)
			mReplacementString = DEFAULT_REPLACEMENT_STRING;
	}


	/**
	 * Run the defined command without any filename replacement.
	 * This executes the command exactly as it was defined.
	 * Returns true if the command was successfully executed
	 * (even if its exit status is nonzero).  The exit status and
	 * other information can be collected after the command finishes.
	 */
	public boolean runCommand()
	{
		//
		// Reset our variables for a new run.
		//
		resetVariables();

		//
		// Make sure there is a command set.
		//
		if ((mCommandArguments == null) || (mCommandArguments.length == 0))
			return setFailed("No command set to be run");

		//
		// See if there is a replacement string defined.
		// If so, then fail since we need a file name.
		//
		if (mReplacementString != null)
			return setFailed("No filename given for command set up to use one");

		//
		// Run the command exactly as specified.
		//
		return internalRunCommand(mCommandArguments);
	}


	/**
	 * Run the defined command using the specified filename.
	 * An argument containing the previously set replacement string
	 * is replaced with the file name.  If there isn't such an argument
	 * then the file name is appended at the end of the command as its
	 * own argument.  Returns true if the command was successfully executed
	 * (even if its exit status is nonzero).  The exit status and other
	 * information can be collected after the command finishes.
	 */
	public boolean runCommand(String fileName)
	{
		//
		// Reset our variables for a new run.
		//
		resetVariables();

		//
		// Make sure there is a command set.
		//
		if ((mCommandArguments == null) || (mCommandArguments.length == 0))
			return setFailed("No command set to be run");

		//
		// If there is no replacement string given then fail.
		//
		if (mReplacementString == null)
			return setFailed("Filename given to command not set up for one");

		//
		// Replace the arguments in the command as necessary.
		//
		String[] newArguments = replaceArguments(fileName);

		//
		// Execute the command.
		//
		return internalRunCommand(newArguments);
	}


	/**
	 * Execute the specified command indicated by the arguments.
	 * Returns true if the command was successfully executed
	 * (no matter what its exit status is).
	 */
	private boolean internalRunCommand(String[] arguments)
	{
		//
		// Catch exceptions during the command.
		//
		try
		{
			//
			// Start the process to be executed.
			//
			Runtime runtime = Runtime.getRuntime();

			mProcess = runtime.exec(arguments);

			//
			// Get the stdio streams for the process.
			//
			InputStream stdoutStream = mProcess.getInputStream();
			InputStream stderrStream = mProcess.getErrorStream();
			OutputStream stdinStream = mProcess.getOutputStream();

			//
			// Close the standard input stream so that the process will
			// get an end of file if it tries to read from its stdin.
			//
			stdinStream.close();

			//
			// Allocate consumer threads to read and discard the output
			// from the process if any.
			//
			InputStreamConsumer stdoutConsumer =
				new InputStreamConsumer(stdoutStream);

			InputStreamConsumer stderrConsumer =
				new InputStreamConsumer(stderrStream);

			Thread stdoutThread = new Thread(stdoutConsumer);
			Thread stderrThread = new Thread(stderrConsumer);

			//
			// Start the consumer threads.
			//
			stdoutThread.start();
			stderrThread.start();

			//
			// Wait for the process to finish.
			//
			mProcess.waitFor();

			//
			// Collect the exit status.
			//
			mExitStatus = mProcess.exitValue();

			mProcess = null;

			//
			// Wait for the consumer threads to finish reading.
			//
			stdoutThread.join();
			stderrThread.join();

			//
			// Save the amount of data that the threads read.
			//
			mStdoutByteCount = stdoutConsumer.getBytesConsumed();
			mStderrByteCount = stderrConsumer.getBytesConsumed();

			//
			// If there was an I/O error reading the output then fail.
			//
			if (stdoutConsumer.isFailed() || stderrConsumer.isFailed())
				return setFailed("Error reading program output");

			//
			// Return success.
			//
			mErrorMessage = null;

			return true;
		}
		catch (ThreadDeath e)
		{
			throw e;
		}
		catch (Throwable e)
		{
			//
			// Try to destroy the process if necessary.
			//
			terminateCommand();

			//
			// Set the original problem, a negative exit status,
			// and return a failure status.
			//
			return setFailed("Problem running command: " + e);
		}
	}


	/**
	 * Try to terminate the command if it is still running.
	 * This is called from another thread to cancel the execution.
	 * This can be called even when there is no command running.
	 * Returns true if the command was successfully terminated.
	 */
	public boolean terminateCommand()
	{
		//
		// Be very safe in trying to destroy the process.
		//
		try
		{
			//
			// Get the process variable and clear it.
			//
			Process process = mProcess;
			mProcess = null;

			//
			// If there is a process then terminate it.
			//
			if (process != null)
			{
				mIsTerminated = true;
				mErrorMessage = "Process forcefully terminated";
				mExitStatus = -1;

				process.destroy();
			}

			return true;
		}
		catch (ThreadDeath e)
		{
			throw e;
		}
		catch (Throwable e)
		{
			return false;
		}
	}


	/**
	 * Return the exit status resulting from running the program.
	 * This is -1 if running the program had failed. 
	 */
	public int getExitStatus()
	{
		return mExitStatus;
	}


	/**
	 * Return the number of bytes that the program wrote to stdout.
	 */
	public int getStdoutByteCount()
	{
		return mStdoutByteCount;
	}


	/**
	 * Return the number of bytes that the program wrote to stderr.
	 */
	public int getStderrByteCount()
	{
		return mStderrByteCount;
	}


	/**
	 * Return an error message that was set on an error.
	 * This is null if there was no error.
	 */
	public String getErrorMessage()
	{
		return mErrorMessage;
	}


	/**
	 * Return the program name which was set for the command to execute.
	 * Returns null if no command is set.
	 */
	public String getProgramName()
	{
		if ((mCommandArguments == null) || (mCommandArguments.length == 0))
			return null;

		return mCommandArguments[0];
	}


	/**
	 * Return whether the command was terminated externally.
	 */
	public boolean isTerminated()
	{
		return mIsTerminated;
	}


	/**
	 * Replace arguments for the command with the file name where
	 * indicated by the replacement string value.  If no argument
	 * contains the replacement string value then the filename is
	 * appended to the end of the command as its own argument.
	 */
	private String[] replaceArguments(String fileName)
	{
		//
		// Allocate a new array for the modified arguments.
		//
		String[] arguments = new String[mCommandArguments.length];

		//
		// Copy the first argument unchanged since it is the
		// program to be executed.
		//
		arguments[0] = mCommandArguments[0];

		//
		// We haven't replaced anything yet.
		//
		boolean isReplaced = false;

		//
		// Examine the arguments and replace any of them which
		// contain the replacement string.  Start after the first
		// argument which is the program to be executed.
		//
		for (int argIndex = 1; argIndex < arguments.length; argIndex++)
		{
			//
			// Get the next original command argument.
			//
			String argument = mCommandArguments[argIndex];

			//
			// Look for the replacement string in the argument.
			//
			int index = argument.indexOf(mReplacementString);

			//
			// If the replacement string was found then replace it
			// with the file name.
			//
			if (index >= 0)
			{
				argument = argument.substring(0, index) + fileName +
					argument.substring(index + mReplacementString.length());

				isReplaced = true;
			}

			//
			// Store the possibly changed argument into the new array.
			//
			arguments[argIndex] = argument;
		}

		//
		// If there was a replacement then return the changed arguments.
		//
		if (isReplaced)
			return arguments;

		//
		// The replacement string was not used so we have to append the
		// filename to the end.  Allocate a slightly larger array to
		// hold the new argument.
		//
		arguments = new String[mCommandArguments.length + 1];

		//
		// Copy all of the old arguments unchanged to the new array.
		//
		System.arraycopy(mCommandArguments, 0, arguments, 0,
			mCommandArguments.length);

		//
		// Add the filename as the last argument and return the array.
		//
		arguments[arguments.length - 1] = fileName;

		return arguments;
	}


	/**
	 * Initialise our data values before starting a new command execution.
	 */
	public void resetVariables()
	{
		mIsTerminated = false;
		mStdoutByteCount = 0;
		mStderrByteCount = 0;
		mErrorMessage = null;
		mExitStatus = -1;
	}


	/**
	 * Set an error message, set the exit status to -1, and return false
	 * for convenience.
	 */
	private boolean setFailed(String message)
	{
		if (message == null)
			message = "No error message specified";

		mErrorMessage = message;
		mExitStatus = -1;

		return false;
	}


	/**
	 * A local class which is used by a thread to consume all of
	 * the data which it can from the specified input stream.
	 * This is used to consume the stdout and stderr output from
	 * the command being executed.
	 */
	private static class InputStreamConsumer implements Runnable
	{
		/**
		 * The input stream to consume.
		 */
		private InputStream mInputStream;

		/**
		 * The amount of data which has been consumed.
		 */
		private int mBytesConsumed;

		/**
		 * Whether there was an I/O error consuming bytes.
		 */
		private boolean mIsFailed;


		/**
		 * Constructor which sets the input stream to be consumed.
		 */
		public InputStreamConsumer(InputStream inputStream)
		{
			mInputStream = inputStream;
		}


		/**
		 * The run method for the Thread to call.
		 */
		@Override
		public void run()
		{
			//
			// Catch exceptions so that we can clean up.
			//
			try
			{
				byte[] buffer = new byte[8192];

				//
				// Keep reading the process output until end of file.
				//
				while (true)
				{
					int count = mInputStream.read(buffer);

					if (count < 0)
						break;

					mBytesConsumed += count;
				}

				mInputStream.close();
				mInputStream = null;
			}
			catch (Exception e)
			{
				mIsFailed = true;
			}
			finally
			{
				Utils.safeClose(mInputStream);
			}
		}


		/**
		 * Return the number of bytes which have been consumed.
		 */
		public int getBytesConsumed()
		{
			return mBytesConsumed;
		}


		/**
		 * Return whether there was an I/O error consuming bytes.
		 */
		public boolean isFailed()
		{
			return mIsFailed;
		}
	}
}
